SystemOrganization addCategory: #'Fuel-Core'!
SystemOrganization addCategory: #'Fuel-Core-Clusters'!
SystemOrganization addCategory: #'Fuel-Core-Mappers'!
SystemOrganization addCategory: #'Fuel-Core-Errors'!
SystemOrganization addCategory: #'Fuel-Collections'!
SystemOrganization addCategory: #'Fuel-Streams'!
SystemOrganization addCategory: #'Fuel-Utilities'!

!ByteSymbol methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:15'!
fuelAccept: aVisitor

	^aVisitor visitSymbol: self! !

!Set methodsFor: '*Fuel' stamp: 'MartinDias 8/13/2011 20:35'!
addIfNotPresent: anObject ifPresentDo: aBlock
	"Include anObject as one of the receiver's elements and then value aBlock, but only if there
	is no such element already. Anwser anObject."

	"Sets in Pharo 1.3 can contain nil, so this implementation is wrong in that version. This is the right one (in Pharo 1.3):
	
	| index |
	index := self scanFor: newObject.
	(array at: index) 
		ifNil: [self atNewIndex: index put: newObject asSetElement]
		ifNotNil: [ aBlock value ].
	^ newObject
	"

	| index |
	anObject ifNil: [self error: 'Sets cannot meaningfully contain nil as an element'].
	index := self findElementOrNil: anObject.
	(array at: index) 
		ifNil: [ self atNewIndex: index put: anObject ]
		ifNotNil: [ aBlock value ].
	^ anObject! !

!CompiledMethod methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:13'!
fuelAccept: aVisitor

	^aVisitor visitCompiledMethod: self! !

!CompiledMethod methodsFor: '*Fuel' stamp: 'MartinDias 10/22/2010 12:56'!
isNamedPrimitive 

	^ self primitive = 117! !

!Character methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:17'!
fuelAccept: aVisitor

	^aVisitor visitCharacter: self! !

!ByteArray methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:12'!
fuelAccept: aVisitor

	^aVisitor visitBytesObject: self! !

Error subclass: #FLClassNotFound
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Errors'!

!FLClassNotFound commentStamp: 'MarianoMartinezPeck 6/5/2011 12:31' prior: 0!
FLClassNotFound is an error that happens when trying to materialize an instance whose class was removed between serialization and materialization time.!

!WriteStream methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 8/25/2011 12:12'!
nextByteStringPut: s 
	"Append the string, s, to the receiver.  Only used by DataStream.  Max size of 64*256*256*256."

	^ self nextStringPut: s! !

!WriteStream methodsFor: '*Fuel' stamp: 'MartinDias 8/13/2011 14:28'!
nextBytesPutAll: aCollection
	"Append the bytes of aCollection to the sequence of bytes accessible 
	by the receiver. Answer aCollection."
 
 	| newEnd |
 	collection class instSpec == aCollection class instSpec ifFalse:
 		[^ super nextPutAll: aCollection ].
 
 	newEnd := position + aCollection size.
 	newEnd > writeLimit ifTrue:
 		[self growTo: newEnd + 10].
 
 	collection replaceFrom: position+1 to: newEnd  with: aCollection startingAt: 1.
 	position := newEnd.
	
	^ aCollection! !

!Behavior methodsFor: '*Fuel' stamp: 'MartinDias 6/13/2011 01:04'!
fuelIgnoredInstanceVariableNames
	"Indicates which variables have to be ignored during serialization."

	^#()! !

!Behavior methodsFor: '*Fuel' stamp: 'MartinDias 3/28/2011 02:39'!
initializeTraitComposition: aTraitComposition

	aTraitComposition assertValidUser: self.
	self traitComposition: aTraitComposition.
	aTraitComposition traits do: [:each | each addUser: self]! !

!Behavior methodsFor: '*Fuel' stamp: 'MartinDias 6/12/2011 23:28'!
instanceVariableNames

	| variables |
	variables := Array new: self instSize.
	self instVarNamesAndOffsetsDo: [ :name :index | variables at: index put: name ].
	^variables! !

!Behavior methodsFor: '*Fuel' stamp: 'MartinDias 3/17/2011 16:36'!
printCompactHierarchy
	"Returns a short description of the class hierarchy"
	
	^ self withAllSuperclasses inject: '' into: [:a :b | b printString, '.', a ]! !

!UndefinedObject methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:42'!
fuelAccept: aVisitor

	^aVisitor visitUndefinedObject: self! !

!OrderedCollection methodsFor: '*Fuel' stamp: 'MartinDias 10/12/2010 16:11'!
hasAnyDuplicate
	"Answers if there is an element added more than one time"

	^ self size ~= self asSet size ! !

!Collection methodsFor: '*Fuel' stamp: 'MartinDias 5/19/2011 23:41'!
addIfNotPresent: anObject ifPresentDo: aBlock
	"Include anObject as one of the receiver's elements and then value aBlock, but only if there
	is no such element already. Anwser anObject."

	(self includes: anObject) 
		ifFalse: [ self add: anObject ]
		ifTrue: [ aBlock value ].
	^ anObject! !

!SystemDictionary methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:40'!
fuelAccept: aVisitor

	^aVisitor visitSystemDictionary: self! !

!WideString methodsFor: '*Fuel' stamp: 'MartinDias 8/15/2011 23:14'!
fuelAccept: aVisitor

	^aVisitor visitWordsArrayedCollection: self! !

!Boolean methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:35'!
fuelAccept: aVisitor

	^aVisitor visitBoolean: self! !

!Trait class methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 6/29/2011 14:38'!
fuelNew
	^ super new! !

!Trait methodsFor: '*Fuel' stamp: 'MartinDias 7/1/2011 04:32'!
basicEnvironment

	^self environment ! !

!Trait methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:33'!
fuelAccept: aVisitor

	^aVisitor visitTrait: self! !

!ArrayedCollection methodsFor: '*Fuel' stamp: 'MartinDias 8/15/2011 23:11'!
fuelAccept: aVisitor

	^(self class isBits and: [self class isWords])
		ifTrue: [	 aVisitor visitWordsArrayedCollection: self ]
		ifFalse: [ super fuelAccept: aVisitor ]! !

!WideSymbol methodsFor: '*Fuel' stamp: 'MartinDias 8/15/2011 23:16'!
fuelAccept: aVisitor

	^aVisitor visitWordsArrayedCollection: self! !

!Float methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:24'!
fuelAccept: aVisitor

	^aVisitor visitFloat: self! !

!Metaclass methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:26'!
fuelAccept: aVisitor

	^aVisitor visitMetaclass: self
! !

!ContextPart class methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 5/22/2011 23:44'!
newFromFrameSize: aFrameSize
	
	^ super basicNew: aFrameSize! !

Stream subclass: #FLBufferedWriteStream
	instanceVariableNames: 'stream buffer position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Streams'!

!FLBufferedWriteStream commentStamp: 'MarianoMartinezPeck 6/5/2011 12:41' prior: 0!
FLBufferedWriteStream is a buffered write stream we use for Fuel serialization. Instead of directly using the stream provided to FLSerializer at creation time by the user, we create an instance of FLBufferedWriteStream for that stream.

MultiByteFileStream has no real buffer and goes to disk too frequently. With FLBufferedWriteStream we keep stuff in a cache and only go to disk when this is full.

The way of using it is jut FLBufferedWriteStream on: aWriteStream. For example:

FLBufferedWriteStream on: (FileDirectory default forceNewFileNamed:  'TestFile') binary

With the message #sizeBuffer: you can set the size of the buffer.

Make sure to always send #flush or #close when you're done, otherwise the last buffer might not yet have been written.
!

!FLBufferedWriteStream class methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
on: writeStream
	^ self basicNew
		on: writeStream;
		yourself! !

!FLBufferedWriteStream class methodsFor: 'convenience' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
on: writeStream do: block
	"Execute block with as argument a ZnBufferedWriteStream on writeStream,
	making sure #flush is called at the end."
	
	| bufferedWriteStream |
	bufferedWriteStream := self on: writeStream.
	block value: bufferedWriteStream.
	bufferedWriteStream flush! !

!FLBufferedWriteStream methodsFor: 'private' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
buffer
	buffer isNil 
		ifTrue: [ self sizeBuffer: self defaultBufferSize ].
	^ buffer! !

!FLBufferedWriteStream methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
bufferFreeSize
	^ self bufferSize - position! !

!FLBufferedWriteStream methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
bufferSize
	^ buffer isNil 
		ifTrue: [ self defaultBufferSize ]
		ifFalse: [ buffer size ]! !

!FLBufferedWriteStream methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
close
	self flushBuffer.
	stream close! !

!FLBufferedWriteStream methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
defaultBufferSize
	^ 8192 "2 raisedTo: 13  "
	

! !

!FLBufferedWriteStream methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
flush
	self flushBuffer.
	stream flush! !

!FLBufferedWriteStream methodsFor: 'private' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
flushBuffer
	position = 0 ifTrue: [ ^ self ].
	position = self bufferSize
		ifTrue: [
			stream nextPutAll: buffer ]
		ifFalse: [
			(stream respondsTo: #next:putAll:startingAt:)
				ifTrue: [ stream next: position putAll: buffer startingAt: 1 ]
				ifFalse: [ stream nextPutAll: (buffer copyFrom: 1 to: position) ] ].
	position := 0! !

!FLBufferedWriteStream methodsFor: 'private' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
flushBufferIfFull
	position = self bufferSize 
		ifTrue: [ self flushBuffer ]
! !

!FLBufferedWriteStream methodsFor: 'printing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
nextInt32Put: int32
	"Write a signed integer to the next 4 bytes"
	| pos |
	self flushBufferIfFull.
	pos := int32 < 0
		ifTrue: [(0-int32) bitInvert32 + 1]
		ifFalse: [int32].
	1 to: 4 do: [:i | self nextPut: (pos digitAt: 5-i)].
	^ int32! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
nextNumber: n put: v 
	"Append to the receiver the argument, v, which is a positive 
	SmallInteger or a LargePositiveInteger, as the next n bytes.
	Possibly pad with leading zeros."
	self flushBufferIfFull.
	1 to: n do: [:i | self nextPut: (v digitAt: n+1-i)].
	^ v
! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
nextPut: object
	self flushBufferIfFull.
	position := position + 1.
	self buffer at: position put: object! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
nextPutAll: collection
	self flushBufferIfFull.
	collection size <= self bufferFreeSize
		ifTrue: [
			self buffer replaceFrom: position + 1 to: position + collection size with: collection.
			position := position + collection size ]
		ifFalse: [
			self flushBuffer.
			collection size > (self bufferSize / 2)
				ifTrue: [ stream nextPutAll: collection ]
				ifFalse: [ self nextPutAll: collection ] ]
! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 8/25/2011 13:18'!
nextStringPut: s 
	"Append the string, s, to the receiver.  Only used by DataStream.  Max size of 64*256*256*256."

	| length |
	self flushBufferIfFull.
	(length := s size) < 192
		ifTrue: [self nextPut: length]
		ifFalse: 
			[self nextPut: (length digitAt: 4)+192.
			self nextPut: (length digitAt: 3).
			self nextPut: (length digitAt: 2).
			self nextPut: (length digitAt: 1)].
	self nextPutAll: s.
	^s! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
nextWordPut: aWord 
	"Append to the receiver an Integer as the next two bytes."
	self flushBufferIfFull.
	self nextPut: ((aWord bitShift: -8) bitAnd: 255).
	self nextPut: (aWord bitAnd: 255).
	^aWord! !

!FLBufferedWriteStream methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
on: writeStream
	stream := writeStream.
	position := 0! !

!FLBufferedWriteStream methodsFor: 'printing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
printOn: aStream
	aStream 
		nextPutAll: 'a '; 
		nextPutAll: self class name! !

!FLBufferedWriteStream methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
sizeBuffer: size
	buffer := (stream isBinary ifTrue: [ ByteArray ] ifFalse: [ String ]) new: size! !

!FLBufferedWriteStream methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
space
	self nextPut: Character space! !

!Stream methodsFor: '*Fuel' stamp: 'MartinDias 8/13/2011 14:29'!
nextBytesPutAll: aCollection
	"Append the bytes of aCollection to the sequence of bytes accessible 
	by the receiver. Answer aCollection."

	^self nextPutAll: aCollection! !

!ClassTrait methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:23'!
fuelAccept: aVisitor

	^aVisitor visitClassTrait: self
! !

!Bitmap methodsFor: '*Fuel' stamp: 'MartinDias 8/15/2011 23:14'!
fuelAccept: aVisitor

	^aVisitor visitWordsArrayedCollection: self! !

!Array methodsFor: '*Fuel' stamp: 'MartinDias 8/15/2011 22:52'!
fuelAccept: aVisitor

	^aVisitor visitVariableObject: self! !

!MethodContext methodsFor: '*Fuel' stamp: 'cwp 7/12/2010 21:42'!
closure: aBlockClosure
	closureOrNil := aBlockClosure! !

!MethodContext methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:28'!
fuelAccept: aVisitor

	^aVisitor visitMethodContext: self! !

!MethodContext methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 5/22/2011 22:11'!
method: aMethod

	method := aMethod ! !

!SmallInteger methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:31'!
fuelAccept: aVisitor

	^aVisitor visitSmallInteger: self! !

!TraitBehavior methodsFor: '*Fuel' stamp: 'MartinDias 3/28/2011 02:45'!
initializeTraitComposition: aTraitComposition

	self traitComposition: aTraitComposition.
	aTraitComposition traits do: [:each | each addUser: self]! !

!TraitBehavior methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 5/21/2011 20:10'!
printCompactHierarchy
	"Returns a short description of the class hierarchy"
	
	^ self withAllSuperclasses inject: '' into: [:a :b | b printString, '.', a ]! !

!HashedCollection methodsFor: '*Fuel' stamp: 'MartinDias 6/13/2011 02:43'!
fuelAfterMaterialization

	self rehash! !

!LargePositiveInteger methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:39'!
fuelAccept: aVisitor

	^aVisitor visitLargeInteger: self! !

!ByteString methodsFor: '*Fuel' stamp: 'MartinDias 8/15/2011 17:20'!
fuelAccept: aVisitor

	^aVisitor visitByteString: self! !

!ClassDescription methodsFor: '*Fuel' stamp: 'MartinDias 7/30/2011 19:16'!
basicInstVarNames
	"Answer nil or an Array of the receiver's instance variable names."

	^instanceVariables! !

!ClassDescription methodsFor: '*Fuel' stamp: 'MartinDias 7/30/2011 19:23'!
basicInstVarNames: anArray

	instanceVariables := anArray! !

!ClassDescription methodsFor: '*Fuel' stamp: 'MartinDias 7/27/2011 22:33'!
fuelInstVarNamesAndOffsetsDo: aBinaryBlock
	"This is part of the interface between the compiler and a class's instance or field names.
	 The class should enumerate aBinaryBlock with the instance variable name strings and
	 their integer offsets.  The order is important. Names evaluated later will override the
	 same names occurring earlier."

	| superInstSize |
	(superInstSize := superclass notNil ifTrue: [superclass instSize] ifFalse: [0]) > 0 ifTrue:
		[superclass fuelInstVarNamesAndOffsetsDo: aBinaryBlock].
	1 to: self instSize - superInstSize do:
		[:i| aBinaryBlock value: (instanceVariables at: i) value: i + superInstSize]! !

!Class methodsFor: '*Fuel' stamp: 'MartinDias 7/30/2011 18:23'!
basicClassPool
	"Answer nil or the dictionary of class variables."

	^classPool! !

!Class methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 6/29/2011 11:04'!
basicEnvironment

	^ environment! !

!Class methodsFor: '*Fuel' stamp: 'MartinDias 7/30/2011 18:23'!
basicSharedPools
	"Answer nil or a Set of the pool dictionaries declared in the receiver."

	^sharedPools! !

!Class methodsFor: '*Fuel' stamp: 'MartinDias 5/14/2011 21:44'!
basicSubclasses

	^ subclasses! !

!Class methodsFor: '*Fuel' stamp: 'MartinDias 5/14/2011 21:45'!
basicSubclasses: anArray

	subclasses := anArray! !

!Class methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:22'!
fuelAccept: aVisitor

	^aVisitor visitClass: self! !

Object subclass: #FLAnalyzer
	instanceVariableNames: 'objectStack mappers firstMapper clusterization'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLAnalyzer commentStamp: 'MartinDias 5/30/2011 03:21' prior: 0!
I am responsible of traverse the graph of references starting on a collection of root objects. I will produce dictionaries that associate each traversed object to a serializer.

An example of use is

	FLAnalyzer newDefault 
		analyze: { 1@2. 3@4 };
		clusteredObjects.
	
The results are used by FLSerializer to serialize the analyzed objects into a stream.!

!FLAnalyzer class methodsFor: 'protected' stamp: 'MartinDias 8/18/2011 17:09'!
fullMappers 
	"Answer a list of mapper classes. They will be applied in order."

	^ { FLGlobalAssociationMapper. FLClassPoolAssociationMapper. FLGlobalMapper. FLGlobalBehaviorMapper. FLDefaultMapper. }! !

!FLAnalyzer class methodsFor: 'protected' stamp: 'MartinDias 8/18/2011 17:09'!
lightMappers
	"Answer a list of mapper classes. They will be applied in order."

	^ {FLLightGlobalMapper. FLDefaultMapper}! !

!FLAnalyzer class methodsFor: 'instance creation' stamp: 'MartinDias 10/26/2010 10:32'!
new

	self error: 'Should use an specific instance creation message.'! !

!FLAnalyzer class methodsFor: 'instance creation' stamp: 'MartinDias 8/18/2011 17:02'!
newDefault
	"Returns an instance with default configuration."

	^self newLight! !

!FLAnalyzer class methodsFor: 'instance creation' stamp: 'MartinDias 8/20/2011 19:31'!
newFull
	"Returns an instance with full configuration."

	^self basicNew 
		initialize;
		initializeMappersWithAll: self fullMappers;
		initializeMappersChain;
		yourself.! !

!FLAnalyzer class methodsFor: 'instance creation' stamp: 'MartinDias 8/20/2011 19:31'!
newLight
	"Returns an instance with light configuration."

	^self basicNew 
		initialize;
		initializeMappersWithAll: self lightMappers;
		initializeMappersChain;
		yourself.! !

!FLAnalyzer methodsFor: 'analyzing' stamp: 'MartinDias 8/7/2011 20:18'!
clusterFor: anObject

	^firstMapper chainedClusterFor: anObject! !

!FLAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 8/8/2011 02:40'!
clusterization

	^ clusterization! !

!FLAnalyzer methodsFor: 'analyzing' stamp: 'MartinDias 8/7/2011 22:57'!
clusterizationOf: theRootObject

	objectStack push: theRootObject.

	[ objectStack isEmpty ] 
		whileFalse: [ self mapAndTrace: objectStack pop ].

	^clusterization! !

!FLAnalyzer methodsFor: 'initialize-release' stamp: 'MartinDias 8/18/2011 16:21'!
initialize

	super initialize.
	objectStack := FLSimpleStack new.
	clusterization := FLClusterization new.
! !

!FLAnalyzer methodsFor: 'initialize-release' stamp: 'MartinDias 8/20/2011 19:28'!
initializeMappersChain
	"Initialize the Chain of Responsibility pattern used in #clusterFor:."
	
	mappers reduceRight: [:left :right | left next: right ].
	firstMapper := mappers first.
	! !

!FLAnalyzer methodsFor: 'initialize-release' stamp: 'MartinDias 8/20/2011 23:16'!
initializeMappersWithAll: mapperClasses
	"Initialize with instances of the received sequence of classes. Note that the order in which they are added is important."

	mappers := mapperClasses collect: [: each | each new ] as: OrderedCollection.! !

!FLAnalyzer methodsFor: 'initialize-release' stamp: 'MartinDias 8/18/2011 16:20'!
internalObjects: aCollection

	self mappers do: [:each | each internalObjects: aCollection].
! !

!FLAnalyzer methodsFor: 'mapping' stamp: 'MartinDias 8/20/2011 22:22'!
mapAndTrace: anObject
	"Map an object to its cluster. Trace its references."

	| aCluster |
	aCluster := self clusterFor: anObject.
	clusterization add: aCluster traceWith: self.
	aCluster add: anObject traceWith: self.
! !

!FLAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 5/30/2011 16:38'!
mappers
	^ mappers! !

!FLAnalyzer methodsFor: 'initialize-release' stamp: 'MartinDias 8/18/2011 16:21'!
reset

	self initialize.! !

!FLAnalyzer methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:11'!
trace: anObject

	objectStack push: anObject! !

!FLAnalyzer methodsFor: 'initialize-release' stamp: 'MartinDias 8/20/2011 23:16'!
when: aCondition substituteBy: aFactory

	mappers addFirst: 
		(FLSubstitutionMapper 
			when: aCondition
			substituteBy: aFactory).
	self initializeMappersChain.! !

Object subclass: #FLCluster
	instanceVariableNames: 'objects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLCluster commentStamp: '<historical>' prior: 0!
I represent a special way of serializing and materializing some specific kind of objects.

Each of my non-abstract subclasses answers to #id with an unique integer, that identifies it. 

Sending me the #forId: message you obtain an instance that corresponds with an specific id. 

Every object responds to #fuelSerializer with its corresponding subclass of me. For example:

	42 fuelSerializer 
	
will return a serializer for that specific instance of SmallInteger.!

!FLCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:23'!
add: anObject traceWith: anAnalyzer.
	"Add an object to the cluster and trace references."

	objects addIfNotPresent: anObject ifPresentDo: [ ^self ].

	self referencesOf: anObject do: [ :aChild | anAnalyzer trace: aChild ].! !

!FLCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:17'!
clusterBucketIn: aClusterization
	"Answers in which clusters bucket place this cluster. This defines the sort order in which the clusters will be ordered. See FLAnalyzer >> clusteredObjects for more information."

	self subclassResponsibility ! !

!FLCluster methodsFor: 'analyzing' stamp: 'MartinDias 4/19/2011 19:53'!
clusterReferencesDo: aBlock
	"Evaluates the received block with each object referenced by the cluster"! !

!FLCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/19/2011 21:40'!
initialize

	super initialize.
	objects := self newCollection.! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:37'!
materializeInstancesStepWith: aMaterializer

	self subclassResponsibility ! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:49'!
materializePostReferencesStepWith: aMaterializer
! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:52'!
materializeReferencesStepWith: aMaterializer
! !

!FLCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/7/2011 01:26'!
newCollection
	"Answer a collection for the objects that correspond to this cluster."

	^IdentitySet new! !

!FLCluster methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 06:59'!
objects
	^ objects! !

!FLCluster methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 06:59'!
objects: anObject
	objects := anObject! !

!FLCluster methodsFor: 'printing' stamp: 'MartinDias 8/20/2011 23:05'!
printNameOn: aStream

	super printOn: aStream! !

!FLCluster methodsFor: 'printing' stamp: 'MartinDias 8/20/2011 23:11'!
printOn: aStream

	self printNameOn: aStream.
	aStream nextPutAll: '->'.
	objects printElementsOn: aStream! !

!FLCluster methodsFor: 'analyzing' stamp: 'MartinDias 3/2/2011 23:07'!
referencesOf: anObject do: aBlock
	"Evaluates the received block with each object referenced by anObject"! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/20/2011 16:32'!
registerIndexesOn: aDictionary

	self objects do: [ :instance | aDictionary at: instance put: aDictionary size + 1 ].! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:35'!
serializeInstancesStepWith: aSerializer

	self subclassResponsibility ! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:35'!
serializePostReferencesStepWith: aSerializer
! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:36'!
serializeReferencesStepWith: aSerializer
! !

FLCluster subclass: #FLIteratingCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLIteratingCluster commentStamp: '<historical>' prior: 0!
I am a template class whose algorithm for serialize a collection of objects consists on delegate the serialization of each individual object to the subclass.!

FLIteratingCluster subclass: #FLBehaviorCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLBehaviorCluster commentStamp: 'MartinDias 7/30/2011 20:31' prior: 0!
I have the common behavior for serializing class and trait behaviors in full detail.!

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:34'!
materializeInstanceWith: aMaterializer
	
	self explicitRequirement! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/29/2011 18:47'!
materializePostReferencesOf: aClassOrTrait with: aMaterializer

	aClassOrTrait initializeTraitComposition: aMaterializer nextEncodedReference.! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:49'!
materializePostReferencesStepWith: aMaterializer

	objects do:	[ :aClassOrTrait | self materializePostReferencesOf: aClassOrTrait with: aMaterializer ]! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/30/2011 20:20'!
materializeReferencesOf: aClassOrTrait with: aMaterializer

	aClassOrTrait methodDictionary: aMaterializer nextEncodedReference.
	aClassOrTrait organization: aMaterializer nextEncodedReference.
	aClassOrTrait basicLocalSelectors: aMaterializer nextEncodedReference.! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:52'!
materializeReferencesStepWith: aMaterializer

	objects do:	[ :aClassOrTrait | self materializeReferencesOf: aClassOrTrait with: aMaterializer ]! !

!FLBehaviorCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/7/2011 00:15'!
newCollection
	"Returns the proper collection for collecting classes and traits. Superclasses should be before of the subclasses."

	^SortedCollection sortBlock: [ :a :b | a printCompactHierarchy <= b printCompactHierarchy ]! !

!FLBehaviorCluster methodsFor: 'analyzing' stamp: 'MartinDias 7/30/2011 20:17'!
referencesOf: aClassOrTrait do: aBlock

	aBlock value: aClassOrTrait methodDictionary.
	aBlock value: aClassOrTrait organization.
	aBlock value: aClassOrTrait traitComposition.
	aBlock value: aClassOrTrait basicLocalSelectors.! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/29/2011 18:45'!
serializePostReferencesOf: aClassOrTrait with: aSerializer

	aSerializer encodeReferenceTo: aClassOrTrait traitComposition.! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:35'!
serializePostReferencesStepWith: aSerializer

	objects do:	[ :aClassOrTrait | self serializePostReferencesOf: aClassOrTrait with: aSerializer ]! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/30/2011 20:18'!
serializeReferencesOf: aClassOrTrait with: aSerializer

	aSerializer encodeReferenceTo: aClassOrTrait methodDictionary.
	aSerializer encodeReferenceTo: aClassOrTrait organization. 
	aSerializer encodeReferenceTo: aClassOrTrait basicLocalSelectors.! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:36'!
serializeReferencesStepWith: aSerializer

	objects do:	[ :aClassOrTrait | self serializeReferencesOf: aClassOrTrait with: aSerializer ]! !

FLBehaviorCluster subclass: #FLClassCluster
	instanceVariableNames: ''
	classVariableNames: 'SubclassSelectors'
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLClassCluster commentStamp: 'MartinDias 7/30/2011 20:31' prior: 0!
I am a cluster for storing classes in full detail.!

!FLClassCluster classSide methodsFor: 'initialize-release' stamp: 'MartinDias 3/29/2011 22:35'!
initialize

	SubclassSelectors := Dictionary new
		at: #normal put: #subclass:instanceVariableNames:classVariableNames:poolDictionaries:category:;
		at: #weak put: #weakSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category:;
		at: #variable put: #variableSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category:;
		at: #bytes put: #variableByteSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category:;
		at: #words put: #variableWordSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category:;
		at: #compiledMethod put: #subclass:instanceVariableNames:classVariableNames:poolDictionaries:category:;
		yourself.! !

!FLClassCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:26'!
clusterBucketIn: aClusterization

	^aClusterization internalClassesAndTraitsBucket! !

!FLClassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:34'!
materializeInstanceWith: aMaterializer

	| aClass superclass category name instanceVariableNames format metaclass |
	metaclass := aMaterializer nextEncodedReference.
	superclass := aMaterializer nextEncodedReference.
	category := aMaterializer nextEncodedReference.
	name := aMaterializer nextEncodedReference.
	format := aMaterializer nextEncodedReference.
	instanceVariableNames := aMaterializer stream nextString subStrings: ' '.

	aClass := metaclass new.	
	aClass 
		superclass: superclass;
		setFormat: format;
		setName: name;
		setInstVarNames: instanceVariableNames;
		basicCategory: category.

	^aClass! !

!FLClassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/30/2011 20:20'!
materializeReferencesOf: aClass with: aMaterializer

	super materializeReferencesOf: aClass with: aMaterializer.

	aClass classPool: aMaterializer nextEncodedReference.
	aClass sharedPools: aMaterializer nextEncodedReference.
	aClass basicSubclasses: aMaterializer nextEncodedReference. 
	aClass environment: aMaterializer nextEncodedReference.
	
	Class class instSize + 1 to: aClass class instSize 
		do: [ :index | aClass instVarAt: index put: aMaterializer nextEncodedReference ]! !

!FLClassCluster methodsFor: 'analyzing' stamp: 'MartinDias 7/30/2011 20:18'!
referencesOf: aClass do: aBlock

	super referencesOf: aClass do: aBlock.

	aBlock value: aClass classSide.
	aBlock value: aClass superclass.
	aBlock value: aClass basicClassPool.
	aBlock value: aClass basicSharedPools. 
	aBlock value: aClass basicSubclasses.
	aBlock value: aClass basicCategory.
	aBlock value: aClass name.
	aBlock value: aClass format.	
	aBlock value: aClass basicEnvironment.
	
	Class class instSize + 1 to: aClass class instSize 
		do: [ :index | aBlock value: (aClass instVarAt: index) ]! !

!FLClassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:28'!
serializeInstance: aClass with: aSerializer

	aSerializer encodeReferenceTo: aClass classSide.
	aSerializer encodeReferenceTo: aClass superclass.
	aSerializer encodeReferenceTo: aClass basicCategory.
	aSerializer encodeReferenceTo: aClass name.
	aSerializer encodeReferenceTo: aClass format.	
	aSerializer stream nextStringPut: aClass instanceVariablesString.
	self flag: #todo. "The array of variable names represents a circularity problem between clusters: if we encode the reference to such array, then the cluster for arrays should be before of Class cluster. But this would disable us to serialize an array of internal classes."
! !

!FLClassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/30/2011 20:18'!
serializeReferencesOf: aClass with: aSerializer

	super serializeReferencesOf: aClass with: aSerializer.

	aSerializer encodeReferenceTo: aClass basicClassPool. 
	aSerializer encodeReferenceTo: aClass basicSharedPools. 
	aSerializer encodeReferenceTo: aClass basicSubclasses.
	aSerializer encodeReferenceTo: aClass basicEnvironment.
 
	Class class instSize + 1 to: aClass class instSize 
		do: [ :index | aSerializer encodeReferenceTo: (aClass instVarAt: index) ]! !

FLBehaviorCluster subclass: #FLClassTraitCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLClassTraitCluster commentStamp: 'MartinDias 7/30/2011 20:30' prior: 0!
I am a cluster for storing class traits in full detail.!

!FLClassTraitCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:25'!
clusterBucketIn: aClusterization

	^aClusterization internalClassTraitsBucket! !

!FLClassTraitCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:34'!
materializeInstanceWith: aMaterializer

	| aTrait baseTrait |
	baseTrait := aMaterializer nextEncodedReference.

	aTrait := ClassTrait for: baseTrait.
	baseTrait classTrait: aTrait.
	^aTrait! !

!FLClassTraitCluster methodsFor: 'analyzing' stamp: 'MartinDias 7/30/2011 15:38'!
referencesOf: aClassSideClassOrTrait do: aBlock

	super referencesOf: aClassSideClassOrTrait do: aBlock.

	aBlock value: aClassSideClassOrTrait baseTrait.! !

!FLClassTraitCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:28'!
serializeInstance: aClassTrait with: aSerializer

	aSerializer encodeReferenceTo: aClassTrait baseTrait.! !

FLBehaviorCluster subclass: #FLMetaclassCluster
	instanceVariableNames: ''
	classVariableNames: 'SubclassSelectors'
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLMetaclassCluster commentStamp: 'MartinDias 7/30/2011 20:30' prior: 0!
I am a cluster for storing metaclasses in full detail.!

!FLMetaclassCluster classSide methodsFor: 'initialize-release' stamp: 'MartinDias 7/29/2011 18:48'!
initialize

	SubclassSelectors := Dictionary new
		at: #normal put: #subclass:instanceVariableNames:classVariableNames:poolDictionaries:category:;
		at: #weak put: #weakSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category:;
		at: #variable put: #variableSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category:;
		at: #bytes put: #variableByteSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category:;
		at: #words put: #variableWordSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category:;
		at: #compiledMethod put: #subclass:instanceVariableNames:classVariableNames:poolDictionaries:category:;
		yourself.! !

!FLMetaclassCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:26'!
clusterBucketIn: aClusterization

	^aClusterization internalMetaclassesBucket! !

!FLMetaclassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:34'!
materializeInstanceWith: aMaterializer

	| superclass instanceVariableNames format |
	superclass := aMaterializer nextEncodedReference.
	instanceVariableNames := aMaterializer stream nextString subStrings: ' '.
	format := aMaterializer stream nextWord.

	^ Metaclass new
		superclass: superclass 
		methodDictionary: MethodDictionary new 
		format: format;
		setInstVarNames: instanceVariableNames;
		yourself
! !

!FLMetaclassCluster methodsFor: 'analyzing' stamp: 'MartinDias 7/29/2011 21:12'!
referencesOf: aMetaclass do: aBlock

	super referencesOf: aMetaclass do: aBlock.
	
	aBlock value: aMetaclass superclass.! !

!FLMetaclassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:28'!
serializeInstance: aMetaclass with: aSerializer

	aSerializer encodeReferenceTo: aMetaclass superclass.
	aSerializer stream nextStringPut: aMetaclass instanceVariablesString.
	aSerializer stream nextWordPut: aMetaclass format.
	! !

FLBehaviorCluster subclass: #FLTraitCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLTraitCluster commentStamp: 'MartinDias 7/30/2011 20:30' prior: 0!
I am a cluster for storing traits in full detail.!

!FLTraitCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:26'!
clusterBucketIn: aClusterization

	^aClusterization internalClassesAndTraitsBucket! !

!FLTraitCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:34'!
materializeInstanceWith: aMaterializer

	| category name classTrait trait |
	category := aMaterializer stream nextString.
	name := aMaterializer stream nextString asSymbol.
	
	trait := Trait fuelNew.
	trait 
		name: name;
		environment: Trait environment;
		traitComposition: {} asTraitComposition;
		basicCategory: category asSymbol.
	
	^ trait! !

!FLTraitCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/30/2011 20:21'!
materializeReferencesOf: aTrait with: aMaterializer

	super materializeReferencesOf: aTrait with: aMaterializer.
	
	aTrait environment: aMaterializer nextEncodedReference.
! !

!FLTraitCluster methodsFor: 'analyzing' stamp: 'MartinDias 7/30/2011 20:18'!
referencesOf: aTrait do: aBlock

	super referencesOf: aTrait do: aBlock.

	aBlock value: aTrait classSide.
	aBlock value: aTrait basicEnvironment.
! !

!FLTraitCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:28'!
serializeInstance: aTrait with: aSerializer

	aSerializer stream nextStringPut: aTrait basicCategory.
	aSerializer stream nextStringPut: aTrait name.
! !

!FLTraitCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/30/2011 20:19'!
serializeReferencesOf: aClass with: aSerializer

	super serializeReferencesOf: aClass with: aSerializer.

	aSerializer encodeReferenceTo: aClass basicEnvironment.
 ! !

FLIteratingCluster subclass: #FLClassPoolAssociationCluster
	instanceVariableNames: 'ownerClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLClassPoolAssociationCluster commentStamp: 'MartinDias 5/30/2011 01:17' prior: 0!
I am a cluster for class variable associations, i.e. bindings that belong to the class pool of a Class included on Smalltalk globals.!

!FLClassPoolAssociationCluster classSide methodsFor: 'instance creation' stamp: 'MartinDias 8/20/2011 06:54'!
for: aClass

	^ self basicNew
		initialize;
		theClass: aClass;
		yourself.! !

!FLClassPoolAssociationCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:24'!
clusterBucketIn: aClusterization

	^aClusterization baselevelBucket! !

!FLClassPoolAssociationCluster methodsFor: 'analyzing' stamp: 'MartinDias 6/5/2011 03:03'!
clusterReferencesDo: aBlock

	aBlock value: ownerClass! !

!FLClassPoolAssociationCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:34'!
materializeInstanceWith: aMaterializer

	| keyName |
	ownerClass := aMaterializer nextEncodedReference.
	keyName := aMaterializer stream nextString.
	^ ownerClass classPool associationAt: keyName asSymbol ! !

!FLClassPoolAssociationCluster methodsFor: 'printing' stamp: 'MartinDias 8/20/2011 23:09'!
printNameOn: aStream
	"Append to the argument, aStream, a sequence of characters that  
	identifies the receiver."
	
	super printNameOn: aStream.
	aStream nextPut: $[.
	self theClass printOn: aStream.
	aStream nextPut: $].
	! !

!FLClassPoolAssociationCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:28'!
serializeInstance: anAssociation with: aSerializer

	aSerializer encodeReferenceTo: ownerClass.
	aSerializer stream nextStringPut: anAssociation key! !

!FLClassPoolAssociationCluster methodsFor: 'accessing' stamp: 'MartinDias 6/5/2011 03:02'!
theClass

	^ ownerClass! !

!FLClassPoolAssociationCluster methodsFor: 'accessing' stamp: 'MartinDias 6/5/2011 03:02'!
theClass: aClass

	ownerClass := aClass! !

FLIteratingCluster subclass: #FLCompiledMethodCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLCompiledMethodCluster commentStamp: 'MartinDias 5/30/2011 01:17' prior: 0!
I am a cluster for CompiledMethod instances.!

!FLCompiledMethodCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:24'!
clusterBucketIn: aClusterization

	^aClusterization baselevelBucket ! !

!FLCompiledMethodCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:34'!
materializeInstanceWith: aMaterializer

	| compiledMethod header bytecodesPlusTrailerSize aReadStream |

	aReadStream := aMaterializer stream.

	header := (aReadStream nextWord << 16) bitOr: aReadStream nextWord.
	bytecodesPlusTrailerSize := aReadStream nextWord.

	compiledMethod := CompiledMethod newMethod: bytecodesPlusTrailerSize header: header.

	compiledMethod initialPC to: compiledMethod size
		do: [ :index | compiledMethod at: index put: aReadStream next ].
		
	^ compiledMethod! !

!FLCompiledMethodCluster methodsFor: 'protected' stamp: 'MartinDias 6/3/2011 21:16'!
materializeLiteralsTo: compiledMethod with: aMaterializer

	1 to: compiledMethod numLiterals 
		do: [ :index | compiledMethod literalAt: index put: aMaterializer nextEncodedReference ]
! !

!FLCompiledMethodCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:52'!
materializeReferencesStepWith: aMaterializer

	objects do: [ :aCompiledMethod | self materializeLiteralsTo: aCompiledMethod with: aMaterializer ]! !

!FLCompiledMethodCluster methodsFor: 'analyzing' stamp: 'MartinDias 3/26/2011 16:27'!
referencesOf: anObject do: aBlock

	1 to: anObject numLiterals
		do: [ :index | aBlock value: (anObject literalAt: index) ]! !

!FLCompiledMethodCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:28'!
serializeInstance: aCompiledMethod with: aSerializer

	| header aWriteStream bytecodesPlusTrailerSize cmSize cmInitialPC |

	aWriteStream := aSerializer stream.

	header := aCompiledMethod header.
	cmSize := aCompiledMethod size.
	cmInitialPC := aCompiledMethod initialPC.
	bytecodesPlusTrailerSize := cmSize - cmInitialPC + 1.

	aWriteStream 
		nextWordPut: (header >> 16);
		nextWordPut: header;
		nextWordPut: bytecodesPlusTrailerSize.

	cmInitialPC to: cmSize
		do: [ :index | aWriteStream nextPut: (aCompiledMethod at: index) ].! !

!FLCompiledMethodCluster methodsFor: 'protected' stamp: 'MartinDias 6/3/2011 20:42'!
serializeLiteralsOf: aCompiledMethod with: aSerializer

	1 to: aCompiledMethod numLiterals 
		do: [ :index | aSerializer encodeReferenceTo: (aCompiledMethod literalAt: index) ]! !

!FLCompiledMethodCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:36'!
serializeReferencesStepWith: aSerializer

	objects do: [ :aCompiledMethod | self serializeLiteralsOf: aCompiledMethod with: aSerializer ]! !

FLCompiledMethodCluster subclass: #FLNamedPrimitiveMethodCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLNamedPrimitiveMethodCluster commentStamp: 'MartinDias 5/30/2011 01:17' prior: 0!
I am a cluster for CompiledMethods which are <primitive:module:> primitives.

For example, Integer >> bitAnd: should be serialized with me.!

!FLNamedPrimitiveMethodCluster methodsFor: 'protected' stamp: 'MartinDias 6/3/2011 21:16'!
materializeLiteralsTo: compiledMethod with: aMaterializer

	| module name |

	module := aMaterializer nextEncodedReference.
	name := aMaterializer nextEncodedReference.
	"The third element of the array is Obsolete. We can put whatever we want since it is not used anymore by the VM. It used to be a session ID. The forth element is the primitive index in a externalPrimitiveTable and it is IMPORTANT to put it in zero. Always. Otherwise there can be errors. Putting a zero means that the VM will not try to directly execute the primitive from the table and it will always try to load the module. It is the safer thing to do. For more details, read the method #primitiveExternalCall in VMMaker"
	compiledMethod literalAt: 1 put: (Array with: module with: name with: 0 with: 0).

	2 to: compiledMethod numLiterals 
		do: [ :index | compiledMethod literalAt: index put: aMaterializer nextEncodedReference ]
! !

!FLNamedPrimitiveMethodCluster methodsFor: 'analyzing' stamp: 'MartinDias 11/11/2010 18:31'!
referencesOf: anObject do: aBlock

	aBlock 
		value: (anObject literalAt: 1) first;
		value: (anObject literalAt: 1) second. 

	2 to: anObject numLiterals
		do: [ :index | aBlock value: (anObject literalAt: index) ]! !

!FLNamedPrimitiveMethodCluster methodsFor: 'protected' stamp: 'MartinDias 6/3/2011 20:42'!
serializeLiteralsOf: aCompiledMethod with: aSerializer

	| primitiveArguments |
	
	primitiveArguments := aCompiledMethod literalAt: 1.
	aSerializer 
		encodeReferenceTo: primitiveArguments first;
		encodeReferenceTo: primitiveArguments second.

	2 to: aCompiledMethod numLiterals 
		do: [ :index | aSerializer encodeReferenceTo: (aCompiledMethod literalAt: index) ]! !

!FLIteratingCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:34'!
materializeInstanceWith: aMaterializer
	"Hook method that each subclass should customize for its special way of materializing"

	^ self subclassResponsibility! !

!FLIteratingCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/28/2011 02:48'!
materializeInstancesStepWith: aMaterializer
	"Template method that let to the subclass the materialization using the received materializer"

	| instanceCount |
	instanceCount := aMaterializer nextEncodedInteger.
	
	objects := OrderedCollection new: instanceCount.
	objects resetTo: 1. "Hack that avoids OrderedCollection>>makeRoomAtLast"
	
	instanceCount timesRepeat: [ 
		objects add: (self materializeInstanceWith: aMaterializer) ].
	
	aMaterializer addAllMaterialized: objects.
! !

!FLIteratingCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:28'!
serializeInstance: anObject with: aSerializer
	"Hook method that each subclass should customize for its special way of serializing"

	self subclassResponsibility ! !

!FLIteratingCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:35'!
serializeInstancesStepWith: aSerializer
	"Template method that let to the subclass the serialization of each instance received"

	aSerializer encodeInteger: self objects size.
	self objects do: [ :instance | self serializeInstance: instance with: aSerializer ]! !

FLIteratingCluster subclass: #FLReferencedClassSideCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLReferencedClassSideCluster commentStamp: 'MartinDias 8/1/2011 02:57' prior: 0!
I am a cluster for those Metaclasses and ClassTrait who are not stored in detail.!

!FLReferencedClassSideCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:25'!
clusterBucketIn: aClusterization

	^aClusterization externalMetaclassesAndClassTraitsBucket ! !

!FLReferencedClassSideCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:34'!
materializeInstanceWith: aMaterializer

	^ aMaterializer nextEncodedReference classSide! !

!FLReferencedClassSideCluster methodsFor: 'analyzing' stamp: 'MartinDias 3/23/2011 18:06'!
referencesOf: aClassOrTraitClassSide do: aBlock

	aBlock value: aClassOrTraitClassSide instanceSide! !

!FLReferencedClassSideCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:28'!
serializeInstance: aClassOrTraitClassSide with: aSerializer

	aSerializer encodeReferenceTo: aClassOrTraitClassSide instanceSide! !

FLCluster subclass: #FLObjectCluster
	instanceVariableNames: 'theClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLObjectCluster commentStamp: 'MartinDias 5/30/2011 01:24' prior: 0!
I implement a generic way of serializing and materializing an object, for those that do not need more special way of doing it.

For example,

	10@20 fuelCluster

returns an instance of a subclass of me.

!

FLObjectCluster subclass: #FLBitsObjectCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLBitsObjectCluster commentStamp: 'MartinDias 8/1/2011 02:59' prior: 0!
I have the common behavior for generic storing and loading variable bits objects.!

!FLBitsObjectCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:25'!
clusterBucketIn: aClusterization

	^aClusterization bitsObjectsBucket! !

FLBitsObjectCluster subclass: #FLByteObjectCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLByteObjectCluster commentStamp: 'MartinDias 8/1/2011 02:59' prior: 0!
I am the generic cluster for storing and loading variable byte objects.!

!FLByteObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/15/2011 20:40'!
materializeInstanceWith: aMaterializer

	| instance size |
	size := aMaterializer nextEncodedInteger.
	instance := theClass basicNew: size.
	aMaterializer stream next: size into: instance.
	^ instance
! !

!FLByteObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/15/2011 20:40'!
serializeInstance: anObject with: aSerializer

	aSerializer encodeInteger: anObject basicSize.
	aSerializer stream nextBytesPutAll: anObject.! !

FLBitsObjectCluster subclass: #FLWordObjectCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLWordObjectCluster commentStamp: 'MartinDias 8/1/2011 02:59' prior: 0!
I am the generic cluster for storing and loading variable word objects.!

FLWordObjectCluster subclass: #FLWordArrayedCollectionCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLWordArrayedCollectionCluster commentStamp: 'MartinDias 8/15/2011 22:35' prior: 0!
I am a cluster for all word-like ArrayedCollection subclasses. 

I use the ArrayedCollection messages #writeOn: and #newFromStream: for serialize and materialize instances.!

!FLWordArrayedCollectionCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/15/2011 22:41'!
materializeInstanceWith: aMaterializer

	^theClass newFromStream: aMaterializer stream! !

!FLWordArrayedCollectionCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/15/2011 22:40'!
serializeInstance: anArrayedCollection with: aSerializer

	anArrayedCollection writeOn: aSerializer stream
! !

!FLWordObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/15/2011 20:45'!
materializeInstanceWith: aMaterializer

	| instance |
	instance := theClass basicNew: aMaterializer nextEncodedInteger.
	aMaterializer stream nextWordsInto: instance.
	^ instance
! !

!FLWordObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/15/2011 20:45'!
serializeInstance: anObject with: aSerializer

	aSerializer encodeInteger: anObject basicSize.
	aSerializer stream nextWordsPutAll: anObject.! !

!FLObjectCluster class methodsFor: 'instance creation' stamp: 'MartinDias 8/20/2011 06:54'!
for: aClass

	^ self basicNew
		initialize;
		theClass: aClass;
		yourself.! !

!FLObjectCluster methodsFor: 'analyzing' stamp: 'MartinDias 5/30/2011 03:45'!
clusterReferencesDo: aBlock

	aBlock value: theClass! !

!FLObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 5/26/2011 23:40'!
materializeInstanceWith: aMaterializer

	self subclassResponsibility ! !

!FLObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 23:01'!
materializeInstancesStepWith: aMaterializer
	"Template method that let to the subclass the materialization using the received materializer"

	| instanceCount |
	theClass := aMaterializer nextEncodedReference.
	instanceCount := aMaterializer nextEncodedInteger.

	objects := OrderedCollection new: instanceCount.
	objects resetTo: 1. "Hack that avoids OrderedCollection>>makeRoomAtLast"

	instanceCount timesRepeat: [ 
		objects add: (self materializeInstanceWith: aMaterializer) ].
	
	aMaterializer addAllMaterialized: objects.
! !

!FLObjectCluster methodsFor: 'printing' stamp: 'MartinDias 8/20/2011 23:10'!
printNameOn: aStream
	"Append to the argument, aStream, a sequence of characters that  
	identifies the receiver."
	
	super printNameOn: aStream.
	aStream nextPut: $[.
	self theClass printOn: aStream.
	aStream nextPut: $].
	! !

!FLObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 5/27/2011 00:05'!
serializeInstance: anObject with: aSerializer

	self subclassResponsibility ! !

!FLObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 23:00'!
serializeInstancesStepWith: aSerializer
	"Template method that let to the subclass the serialization of each instance received"

	aSerializer encodeReferenceTo: theClass.
	aSerializer encodeInteger: objects size.
	objects do: [ :instance | self serializeInstance: instance with: aSerializer ]! !

!FLObjectCluster methodsFor: 'accessing' stamp: 'MartinDias 5/30/2011 03:42'!
theClass

	^theClass ! !

!FLObjectCluster methodsFor: 'accessing' stamp: 'MartinDias 5/30/2011 03:43'!
theClass: aClass

	theClass := aClass ! !

FLObjectCluster subclass: #FLPointerObjectCluster
	instanceVariableNames: 'variablesMapping'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLPointerObjectCluster commentStamp: 'MartinDias 8/1/2011 03:00' prior: 0!
I have the common behavior for storing and loading pointer objects.!

FLPointerObjectCluster subclass: #FLFixedObjectCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLFixedObjectCluster commentStamp: 'MartinDias 5/30/2011 01:25' prior: 0!
I am a generic cluster for objects without indexable variables.!

!FLFixedObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 5/30/2011 03:42'!
materializeInstanceWith: aMaterializer

	^theClass basicNew! !

!FLFixedObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 5/27/2011 00:05'!
serializeInstance: anObject with: aSerializer

! !

!FLPointerObjectCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:24'!
clusterBucketIn: aClusterization

	^aClusterization baselevelBucket ! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 23:03'!
materializeInstancesStepWith: aMaterializer
	"Template method that let to the subclass the materialization using the received materializer"

	super materializeInstancesStepWith: aMaterializer.
		
	variablesMapping := FLVariablesMapping for: theClass.
	variablesMapping materializeMappingFrom: aMaterializer stream.! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:55'!
materializePostReferencesStepWith: aMaterializer  

	objects do: [ :anObject | anObject fuelAfterMaterialization ]! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 19:09'!
materializeReferencesOf: anObject with: aMaterializer

	variablesMapping materializeReferencesOf: anObject with: aMaterializer! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:56'!
materializeReferencesStepWith: aMaterializer

	objects do: [ :anObject | self materializeReferencesOf: anObject with: aMaterializer ]! !

!FLPointerObjectCluster methodsFor: 'analyzing' stamp: 'MartinDias 7/27/2011 22:32'!
referencesOf: anObject do: aBlock

	| ignoredInstanceVariableNames |
	ignoredInstanceVariableNames := theClass fuelIgnoredInstanceVariableNames.
	
	theClass fuelInstVarNamesAndOffsetsDo: [:name :index | 
		(ignoredInstanceVariableNames includes: name) 
			ifFalse: [ aBlock value: (anObject instVarAt: index) ]]
! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:35'!
serializeInstancesStepWith: aSerializer
	"Template method that let to the subclass the serialization of each instance received"

	super serializeInstancesStepWith: aSerializer.
	
	variablesMapping := FLVariablesMapping for: theClass.
	variablesMapping serializeMappingOn: aSerializer stream.! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 19:06'!
serializeReferencesOf: anObject with: aSerializer
	
	variablesMapping serializeReferencesOf: anObject with: aSerializer
! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:55'!
serializeReferencesStepWith: aSerializer

	objects do: [ :anObject | self serializeReferencesOf: anObject with: aSerializer ]! !

FLPointerObjectCluster subclass: #FLVariableObjectCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLVariableObjectCluster commentStamp: 'MartinDias 5/30/2011 01:25' prior: 0!
I am a generic cluster for objects with indexable variables.!

FLVariableObjectCluster subclass: #FLMethodContextCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLMethodContextCluster commentStamp: 'MartinDias 5/30/2011 01:18' prior: 0!
I am a cluster for MethodContexts.!

!FLMethodContextCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/27/2011 23:49'!
materializeInstanceWith: aMaterializer

	^ theClass newFromFrameSize: aMaterializer stream nextWord
! !

!FLMethodContextCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/27/2011 21:37'!
serializeInstance: aMethodContext with: aSerializer

	aSerializer stream nextWordPut: aMethodContext method frameSize.
! !

!FLVariableObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/1/2011 01:35'!
materializeInstanceWith: aMaterializer

	^theClass basicNew: aMaterializer nextEncodedReference! !

!FLVariableObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 19:09'!
materializeReferencesOf: anObject with: aMaterializer

	super materializeReferencesOf: anObject with: aMaterializer.

	1 to: anObject basicSize 
		do: [ :index | anObject basicAt: index put: aMaterializer nextEncodedReference ]! !

!FLVariableObjectCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/1/2011 01:36'!
referencesOf: anObject do: aBlock

	super referencesOf: anObject do: aBlock.

	aBlock value: anObject basicSize.

	1 to: anObject basicSize 
		do: [ :index | aBlock value: (anObject basicAt: index) ]! !

!FLVariableObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/1/2011 01:35'!
serializeInstance: anObject with: aSerializer

	aSerializer encodeReferenceTo: anObject basicSize! !

!FLVariableObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 19:06'!
serializeReferencesOf: anObject with: aSerializer

	super serializeReferencesOf: anObject with: aSerializer.
	
	1 to: anObject basicSize do: [ :index |
		aSerializer encodeReferenceTo: (anObject basicAt: index) ]! !

FLCluster subclass: #FLPrimitiveCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLPrimitiveCluster commentStamp: 'MartinDias 5/30/2011 01:27' prior: 0!
I am a template class whose algorithm for serialize a collection of objects is to delegate the serialization of each individual object to the subclass. 

(TODO) My main difference with FLIteratingCluster is that my method hooks only receive a stream and not the complete instance of FLSerializer, because they have not need. But possibly there is no reason for being different classes.!

FLPrimitiveCluster subclass: #FLByteStringCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLByteStringCluster commentStamp: 'MartinDias 5/30/2011 01:27' prior: 0!
I am a cluster for ByteString.!

!FLByteStringCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/15/2011 17:18'!
materializeFrom: aReadStream

	^ aReadStream nextString! !

!FLByteStringCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 8/25/2011 13:20'!
serialize: aSymbol on: aWriteStream

	aWriteStream nextStringPut: aSymbol! !

FLPrimitiveCluster subclass: #FLCharacterCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLCharacterCluster commentStamp: 'MartinDias 5/30/2011 01:27' prior: 0!
I am a cluster for Character.!

!FLCharacterCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2010 11:06'!
materializeFrom: aReadStream

	^Character value: aReadStream next! !

!FLCharacterCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 11/22/2010 17:33'!
serialize: aCharacter on: aWriteStream

	aWriteStream nextPut: aCharacter codePoint! !

FLPrimitiveCluster subclass: #FLFloatCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLFloatCluster commentStamp: 'MartinDias 5/30/2011 01:27' prior: 0!
I am a cluster for 8-byte Floats. 

Implementation extracted from ReferenceStream.
!

!FLFloatCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/21/2010 13:27'!
materializeFrom: aReadStream

	^ (Float new: 2)
		at: 1 put: (aReadStream nextNumber: 4);
		at: 2 put: (aReadStream nextNumber: 4);
		yourself.
! !

!FLFloatCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/21/2010 10:39'!
serialize: aFloat on: aWriteStream

	aWriteStream 
		nextNumber: 4 put: (aFloat at: 1);
		nextNumber: 4 put: (aFloat at: 2).! !

FLPrimitiveCluster subclass: #FLGlobalAssociationCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLGlobalAssociationCluster commentStamp: 'MartinDias 5/30/2011 01:28' prior: 0!
I am a cluster for variable bindings that belong to Smalltalk global dictionary.

For example, 

	Smalltalk globals associationAt: #Transcript 
	
could be serialized and materialized by me.!

!FLGlobalAssociationCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:24'!
clusterBucketIn: aClusterization

	^aClusterization baselevelBucket ! !

!FLGlobalAssociationCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/23/2011 15:25'!
materializeFrom: aReadStream

	^ Smalltalk globals 
		associationDeclareAt: aReadStream nextString asSymbol
! !

!FLGlobalAssociationCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/18/2010 12:10'!
serialize: anAssociation on: aWriteStream
	
	aWriteStream nextStringPut: anAssociation key! !

FLPrimitiveCluster subclass: #FLGlobalClassCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLGlobalClassCluster commentStamp: 'MartinDias 5/30/2011 01:28' prior: 0!
I am a cluster for classes that should be present when materializing. In other words, it only serializes a reference to the class, using its name.!

!FLGlobalClassCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:25'!
clusterBucketIn: aClusterization

	^aClusterization externalClassesTraitsAndGlobalsBucket ! !

!FLGlobalClassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 5/30/2011 03:48'!
materializeFrom: aReadStream
	| className |
	className := aReadStream nextString asSymbol .
	^ Smalltalk globals 
		at: className
		ifAbsent: [ FLClassNotFound new signal: 'Class', className, ' not found at materialization' ]! !

!FLGlobalClassCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 5/12/2011 10:32'!
serialize: aClass on: aWriteStream
	
	aWriteStream nextStringPut: aClass name ! !

FLPrimitiveCluster subclass: #FLGlobalCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLGlobalCluster commentStamp: 'MartinDias 5/30/2011 01:28' prior: 0!
I am a cluster for known objects that belong to Smalltalk global dictionary.

For example, Transcript could be serialized and materialized by me.!

!FLGlobalCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:25'!
clusterBucketIn: aClusterization

	^aClusterization externalClassesTraitsAndGlobalsBucket ! !

!FLGlobalCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/19/2010 14:52'!
materializeFrom: aReadStream

	^ Smalltalk globals 
		at: aReadStream nextString asSymbol 
		ifAbsent: [ self error: 'Smalltalk global not found' ]! !

!FLGlobalCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 5/12/2011 10:13'!
serialize: anObject on: aWriteStream
	
	aWriteStream 
		nextStringPut: (Smalltalk globals keyAtValue: anObject).! !

!FLPrimitiveCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:26'!
clusterBucketIn: aClusterization

	^aClusterization primitiveObjectsBucket ! !

!FLPrimitiveCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2010 13:30'!
materializeFrom: aReadStream
	"Hook method that each subclass should customize for its special way of materializing"
	
	self subclassResponsibility ! !

!FLPrimitiveCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:59'!
materializeInstancesStepWith: aMaterializer 
	"Template method that let to the subclass the materialization using the received materializer"
	
	| instanceCount stream |
	instanceCount := aMaterializer nextEncodedInteger.

	objects := OrderedCollection new: instanceCount.
	objects resetTo: 1. "Hack that avoids OrderedCollection>>makeRoomAtLast"

	stream := aMaterializer stream.
	instanceCount timesRepeat: [ 
		objects add: (self materializeFrom: stream) ].

	aMaterializer addAllMaterialized: objects.
! !

!FLPrimitiveCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2010 13:29'!
serialize: anObject on: aWriteStream
	"Hook method that each subclass should customize for its special way of serializing"
	
	self subclassResponsibility! !

!FLPrimitiveCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:35'!
serializeInstancesStepWith: aSerializer
	"Template method that let to the subclass the serialization of each instance received"
	
	aSerializer encodeInteger: self objects size.
	self objects do: [ :instance | self serialize: instance on: aSerializer stream ]! !

FLPrimitiveCluster subclass: #FLSmallIntegerCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLSmallIntegerCluster commentStamp: '<historical>' prior: 0!
I don't have so much sense as a class, but I know which of my subclasses corresponds to each SmallInteger.!

FLSmallIntegerCluster subclass: #FLNegativeSmallIntegerCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLNegativeSmallIntegerCluster commentStamp: 'MartinDias 5/30/2011 01:29' prior: 0!
I am a cluster for negative SmallInteger instances.!

!FLNegativeSmallIntegerCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/12/2010 21:39'!
materializeFrom: aReadStream

	| hi mid1 mid2 lo |
	hi := aReadStream next.
	mid2 := aReadStream next.
	mid1 := aReadStream next.
	lo := aReadStream next.
	^ ((hi bitShift: 24) + (mid2 bitShift: 16) + (mid1 bitShift: 8) + lo) negated! !

!FLNegativeSmallIntegerCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/12/2010 10:38'!
serialize: anInteger on: aWriteStream

	| aPositiveInteger |
	aPositiveInteger := anInteger abs.
	aWriteStream nextPut: (aPositiveInteger bitShift: -24).
	aWriteStream nextPut: ((aPositiveInteger bitShift: -16) bitAnd: 255).
	aWriteStream nextPut: ((aPositiveInteger bitShift: -8) bitAnd: 255).
	aWriteStream nextPut: (aPositiveInteger bitAnd: 255).! !

FLSmallIntegerCluster subclass: #FLPositiveSmallIntegerCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLPositiveSmallIntegerCluster commentStamp: 'MartinDias 5/30/2011 01:29' prior: 0!
I am a cluster for positive SmallInteger instances.!

!FLPositiveSmallIntegerCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/15/2010 10:51'!
materializeFrom: aReadStream

	| hi mid1 mid2 lo |
	hi := aReadStream next.
	mid2 := aReadStream next.
	mid1 := aReadStream next.
	lo := aReadStream next.
	^ (((hi bitShift: 24) bitXor: (mid2 bitShift: 16)) bitXor: (mid1 bitShift: 8)) bitXor: lo! !

!FLPositiveSmallIntegerCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/24/2010 00:48'!
serialize: anInteger on: aWriteStream

	aWriteStream nextPut: (anInteger bitShift: -24).
	aWriteStream nextPut: ((anInteger bitShift: -16) bitAnd: 255).
	aWriteStream nextPut: ((anInteger bitShift: -8) bitAnd: 255).
	aWriteStream nextPut: (anInteger bitAnd: 255).! !

!FLSmallIntegerCluster class methodsFor: 'instance creation' stamp: 'MartinDias 8/11/2011 02:45'!
for: anSmallInteger
	"Returns the right subclass instance"

	((anSmallInteger bitAnd: 16rFFFF) = anSmallInteger)
		ifTrue: [ ^ FLUInt16Cluster new ].

	anSmallInteger > 0 
		ifTrue: [ ^ FLPositiveSmallIntegerCluster new ]
		ifFalse: [ ^ FLNegativeSmallIntegerCluster new ]! !

FLSmallIntegerCluster subclass: #FLUInt16Cluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLUInt16Cluster commentStamp: 'MartinDias 5/30/2011 01:29' prior: 0!
I am a cluster for SmallInteger instances between 0 and 16rFFFF.!

!FLUInt16Cluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 12/22/2010 12:20'!
materializeFrom: aReadStream

	| hi lo |
	hi := aReadStream next.
	lo := aReadStream next.
	^ (hi bitShift: 8) bitXor: lo! !

!FLUInt16Cluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 12/22/2010 13:16'!
serialize: anInteger on: aWriteStream

	aWriteStream nextPut: (anInteger bitShift: -8).
	aWriteStream nextPut: (anInteger bitAnd: 255).! !

FLPrimitiveCluster subclass: #FLSymbolCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLSymbolCluster commentStamp: 'MartinDias 5/30/2011 01:29' prior: 0!
I am a cluster for Symbol instances.!

!FLSymbolCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/26/2010 14:22'!
materializeFrom: aReadStream

	^ aReadStream nextString asSymbol! !

!FLSymbolCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 8/25/2011 13:21'!
serialize: aSymbol on: aWriteStream

	aWriteStream nextStringPut: aSymbol! !

FLPrimitiveCluster subclass: #FLWellKnownObjectsCluster
	instanceVariableNames: ''
	classVariableNames: 'WellKnownObjectsDictionary'
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLWellKnownObjectsCluster commentStamp: 'MartinDias 8/1/2011 03:00' prior: 0!
I am a cluster for some well known objects like nil, true and false. They have a special treatment because they are so common in the image.!

!FLWellKnownObjectsCluster classSide methodsFor: 'class initialization' stamp: 'MartinDias 7/1/2011 04:08'!
initialize 

	WellKnownObjectsDictionary := Dictionary new
		add: 1 -> nil;
		add: 2 -> true;
		add: 3 -> false;
		add: 4 -> Smalltalk globals;
		yourself.! !

!FLWellKnownObjectsCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/19/2010 12:21'!
materializeFrom: aReadStream
	
	^ WellKnownObjectsDictionary at: aReadStream next
	! !

!FLWellKnownObjectsCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/7/2011 00:12'!
newCollection

	^OrderedCollection new! !

!FLWellKnownObjectsCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/19/2010 13:30'!
serialize: aWellKnownObject on: aWriteStream

	aWriteStream nextPut: (WellKnownObjectsDictionary keyAtValue: aWellKnownObject)! !

FLCluster subclass: #FLSubstitutionCluster
	instanceVariableNames: 'substitutions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLSubstitutionCluster commentStamp: 'MartinDias 8/20/2011 23:45' prior: 0!
I am a cluster for substituted objects. 

For example, suppose you want to substitute instances of WriteStream by nil. In such case, the mapper will associate every WriteStream instance to me, and my factory block will value to nil with any argument.
!

FLSubstitutionCluster subclass: #FLFactorySubstitutionCluster
	instanceVariableNames: 'factory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLFactorySubstitutionCluster commentStamp: 'MartinDias 8/21/2011 00:56' prior: 0!
I am a cluster for substituted objects. 

For example, suppose you want to substitute every integer by its string representation. In such case you should configure the analyzerthrough:

anAnalyzer when: [:x | x isInteger] substituteBy: [:x | x printString].

In this way, when you serialize #(1 2), you will then materialize #('1' '2')!

!FLFactorySubstitutionCluster classSide methodsFor: 'instance creation' stamp: 'MartinDias 8/21/2011 00:47'!
with: aFactory

	^super basicNew
		factory: aFactory;
		initialize;
		yourself.! !

!FLFactorySubstitutionCluster methodsFor: 'accessing' stamp: 'MartinDias 8/21/2011 00:46'!
factory
	^ factory! !

!FLFactorySubstitutionCluster methodsFor: 'accessing' stamp: 'MartinDias 8/21/2011 00:46'!
factory: anObject
	factory := anObject! !

!FLFactorySubstitutionCluster methodsFor: 'accessing' stamp: 'MartinDias 8/21/2011 00:45'!
substitutionFor: anObject
	
	^factory value: anObject! !

FLSubstitutionCluster subclass: #FLHookSubstitutionCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLHookSubstitutionCluster methodsFor: 'accessing' stamp: 'MartinDias 8/21/2011 00:46'!
substitutionFor: anObject
	
	^anObject fuelSubstitution! !

!FLSubstitutionCluster methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 22:46'!
add: anObject traceWith: anAnalyzer

	substitutions 
		at: anObject 
		ifAbsent: [ 
			| substitution |
			substitution := self substitutionFor: anObject.
			substitutions at: anObject put: substitution.
			anAnalyzer trace: substitution ] ! !

!FLSubstitutionCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:50'!
clusterBucketIn: aClusterization

	^aClusterization substitutionsBucket! !

!FLSubstitutionCluster methodsFor: 'initialize-release' stamp: 'MartinDias 8/20/2011 23:17'!
initialize

	super initialize.
	substitutions := IdentityDictionary new.! !

!FLSubstitutionCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:37'!
materializeInstancesStepWith: aMaterializer

	| instanceCount |
	instanceCount := aMaterializer nextEncodedInteger.

	"workaround: fill with instanceCount elements"
	aMaterializer addAllMaterialized: (Array new: instanceCount).! !

!FLSubstitutionCluster methodsFor: 'printing' stamp: 'MartinDias 8/20/2011 23:40'!
printOn: aStream

	self printNameOn: aStream.
	aStream nextPutAll: '->'.
	substitutions printElementsOn: aStream! !

!FLSubstitutionCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/20/2011 23:18'!
registerIndexesOn: aDictionary

	substitutions keysAndValuesDo: [ :anObject :theSubstitution | 
		| substitutionIndex |
		substitutionIndex := aDictionary at: theSubstitution.
		aDictionary at: anObject put: substitutionIndex. ].! !

!FLSubstitutionCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:35'!
serializeInstancesStepWith: aSerializer

	aSerializer encodeInteger: self objects size.
! !

!FLSubstitutionCluster methodsFor: 'accessing' stamp: 'MartinDias 8/21/2011 00:45'!
substitutionFor: anObject
	
	self subclassResponsibility! !

Object subclass: #FLClusterization
	instanceVariableNames: 'baselevelBucket bitsObjectsBucket externalClassesTraitsAndGlobalsBucket externalMetaclassesAndClassTraitsBucket internalClassTraitsBucket internalClassesAndTraitsBucket internalMetaclassesBucket primitiveObjectsBucket substitutionsBucket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLClusterization methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:16'!
add: aCluster traceWith: anAnalyzer

	| clusterBucket |
	clusterBucket := aCluster clusterBucketIn: self.
	
	clusterBucket 
		addIfNotPresent: aCluster 
		ifPresentDo: [ ^self ].

	aCluster clusterReferencesDo: [ :aChild | anAnalyzer trace: aChild ]! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 8/20/2011 22:27'!
baselevelBucket
	
	^ baselevelBucket! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 8/20/2011 22:28'!
bitsObjectsBucket
	
	^ bitsObjectsBucket! !

!FLClusterization methodsFor: 'accessing' stamp: 'MartinDias 8/24/2011 20:32'!
clusterBuckets
	"Answer a new collection with all the clusters that have been collected in trace. Note that order is imporant."

	^ OrderedCollection new 
		add: externalClassesTraitsAndGlobalsBucket;
		add: externalMetaclassesAndClassTraitsBucket;
		add: primitiveObjectsBucket;
		add: bitsObjectsBucket;
		add: internalMetaclassesBucket;
		add: internalClassesAndTraitsBucket;
		add: internalClassTraitsBucket;
		add: baselevelBucket;
		add: substitutionsBucket;
		yourself.! !

!FLClusterization methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 19:47'!
clusters
	"Returns collection with all the clustered objects, in Association instances."
	
	^self clusterBuckets inject: OrderedCollection new into: [:c :d | c addAll: d. c ].
! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 8/20/2011 22:28'!
externalClassesTraitsAndGlobalsBucket
	
	^ externalClassesTraitsAndGlobalsBucket ! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 8/20/2011 22:28'!
externalMetaclassesAndClassTraitsBucket
	
	^ externalMetaclassesAndClassTraitsBucket! !

!FLClusterization methodsFor: 'initialize-release' stamp: 'MartinDias 8/20/2011 22:52'!
initialize

	primitiveObjectsBucket := IdentitySet new.
	baselevelBucket := IdentitySet new.
	externalClassesTraitsAndGlobalsBucket := IdentitySet new.
	internalClassesAndTraitsBucket := IdentitySet new.
	internalMetaclassesBucket := IdentitySet new.
	externalMetaclassesAndClassTraitsBucket := IdentitySet new.
	internalClassTraitsBucket := IdentitySet new.
	bitsObjectsBucket := IdentitySet new.
	substitutionsBucket := IdentitySet new.
! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 8/20/2011 22:28'!
internalClassTraitsBucket
	
	^ internalClassTraitsBucket! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 8/20/2011 22:28'!
internalClassesAndTraitsBucket
	
	^ internalClassesAndTraitsBucket ! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 8/20/2011 22:28'!
internalMetaclassesBucket
	
	^ internalMetaclassesBucket! !

!FLClusterization methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 22:49'!
objectCount

	^self clusterBuckets sum: [ :aClusterBucket | 
		aClusterBucket
			inject: 0 
			into: [ :sum :aCluster | sum + aCluster objects size ] ].! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 8/20/2011 22:28'!
primitiveObjectsBucket
	
	^ primitiveObjectsBucket! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 8/20/2011 22:52'!
substitutionsBucket
	
	^ substitutionsBucket! !

Object subclass: #FLMapper
	instanceVariableNames: 'next'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Mappers'!

!FLMapper commentStamp: 'MartinDias 8/11/2011 03:09' prior: 0!
The purpose of my hierarchy is to map objects with clusters.!

FLMapper subclass: #FLClassPoolAssociationMapper
	instanceVariableNames: 'internalClasses classPoolBindings clustersByClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Mappers'!

!FLClassPoolAssociationMapper commentStamp: 'MartinDias 5/30/2011 01:14' prior: 0!
I map variable class variable associations, i.e. bindings that belong to the class pool of a Class existent on Smalltalk globals. !

!FLClassPoolAssociationMapper methodsFor: 'accessing' stamp: 'MartinDias 8/18/2011 20:31'!
accepts: anObject

	^ anObject isVariableBinding and: [ (self isClassVariableBinding: anObject) and: [ self isExternal: anObject ] ]! !

!FLClassPoolAssociationMapper methodsFor: 'accessing' stamp: 'MartinDias 8/7/2011 16:39'!
clusterFor: anObject
	
	| theClass |
	theClass := classPoolBindings at: anObject.
	
	^clustersByClass 
		at: theClass 
		ifAbsentPut: [ FLClassPoolAssociationCluster for: theClass ]! !

!FLClassPoolAssociationMapper methodsFor: 'initialize-release' stamp: 'MartinDias 8/18/2011 20:32'!
initialize

	super initialize.
	clustersByClass := IdentityDictionary new.
	internalClasses := OrderedCollection new.
	classPoolBindings := IdentityDictionary new.
	Smalltalk allClassesAndTraitsDo: [ :aClass | 
				aClass classPool associationsDo: [ :aBinding | 
					classPoolBindings at: aBinding put: aClass ] ].! !

!FLClassPoolAssociationMapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 7/6/2011 14:31'!
internalObjects: internalGlobalObjects

	internalClasses := internalGlobalObjects select: [ :anObject | anObject isBehavior ]! !

!FLClassPoolAssociationMapper methodsFor: 'private' stamp: 'eem 6/8/2011 11:50'!
isClassVariableBinding: anObject
	^classPoolBindings includesKey: anObject! !

!FLClassPoolAssociationMapper methodsFor: 'private' stamp: 'eem 6/8/2011 11:50'!
isExternal: anObject
	^ (internalClasses includes: (classPoolBindings at: anObject)) not! !

FLMapper subclass: #FLDefaultMapper
	instanceVariableNames: 'wellKnownObjectCluster characterCluster classCluster classTraitCluster compiledMethodCluster namedCompiledMethodCluster floatCluster metaclassCluster shortCluster positiveSmallIntegerCluster negativeSmallIntegerCluster symbolCluster traitCluster byteObjectClusters variableObjectClusters fixedObjectClusters wordObjectClusters byteStringCluster wordArrayedCollectionsClusters hookSubstitutionCluster'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Mappers'!

!FLDefaultMapper commentStamp: 'MartinDias 8/7/2011 12:24' prior: 0!
I know how to map an object to its default cluster. I can map every object.!

!FLDefaultMapper methodsFor: 'accessing' stamp: 'MartinDias 8/7/2011 12:22'!
accepts: anObject

	^ true! !

!FLDefaultMapper methodsFor: 'accessing' stamp: 'MartinDias 8/11/2011 03:02'!
chainedClusterFor: anObject

	^anObject fuelAccept: self! !

!FLDefaultMapper methodsFor: 'accessing' stamp: 'MartinDias 8/7/2011 12:47'!
clusterFor: anObject

	^anObject fuelAccept: self
! !

!FLDefaultMapper methodsFor: 'initialize-release' stamp: 'MartinDias 8/21/2011 00:49'!
initialize

	super initialize.
	fixedObjectClusters := IdentityDictionary new.
	variableObjectClusters := IdentityDictionary new.
	byteObjectClusters := IdentityDictionary new.
	wordObjectClusters := IdentityDictionary new.
	wordArrayedCollectionsClusters := IdentityDictionary new.
	
	wellKnownObjectCluster := FLWellKnownObjectsCluster new.
	characterCluster := FLCharacterCluster new.
	classCluster := FLClassCluster new.
	classTraitCluster := FLClassTraitCluster new.
	compiledMethodCluster := FLCompiledMethodCluster new.
	namedCompiledMethodCluster := FLNamedPrimitiveMethodCluster new.
	floatCluster := FLFloatCluster new.
	metaclassCluster := FLMetaclassCluster new.
	shortCluster := FLUInt16Cluster new.
	positiveSmallIntegerCluster := FLPositiveSmallIntegerCluster new.
	negativeSmallIntegerCluster := FLNegativeSmallIntegerCluster new.
	symbolCluster := FLSymbolCluster new.
	traitCluster := FLTraitCluster new.
	byteStringCluster := FLByteStringCluster new.
	hookSubstitutionCluster := FLHookSubstitutionCluster new.
! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:36'!
visitBoolean: aBoolean

	^wellKnownObjectCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/15/2011 17:20'!
visitByteString: aSymbol

	^byteStringCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 03:04'!
visitBytesObject: anObject

	^byteObjectClusters
		at: anObject class
		ifAbsentPut: [ FLByteObjectCluster for: anObject class ]! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 03:06'!
visitCharacter: aCharacter

	^aCharacter isOctetCharacter 
		ifTrue: [ characterCluster ]
		ifFalse: [ 
			fixedObjectClusters
				at: aCharacter class
				ifAbsentPut: [ FLFixedObjectCluster for: aCharacter class ]]! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:38'!
visitClass: aClass

	^classCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:38'!
visitClassTrait: aClassTrait

	^classTraitCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:39'!
visitCompiledMethod: aCompiledMethod

	^aCompiledMethod isNamedPrimitive
		ifTrue:  [ namedCompiledMethodCluster ]
		ifFalse: [ compiledMethodCluster ]
! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 03:05'!
visitFixedObject: anObject

	^fixedObjectClusters
		at: anObject class
		ifAbsentPut: [ FLFixedObjectCluster for: anObject class ]! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:39'!
visitFloat: aFloat

	^floatCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 03:05'!
visitLargeInteger: anObject

	^byteObjectClusters
		at: anObject class
		ifAbsentPut: [ FLByteObjectCluster for: anObject class ]! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:39'!
visitMetaclass: aMetaclass

	^metaclassCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 03:05'!
visitMethodContext: aMethodContext

	^variableObjectClusters
		at: aMethodContext class
		ifAbsentPut: [ FLMethodContextCluster for: aMethodContext class ]
! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:42'!
visitSmallInteger: aSmallInteger

	((aSmallInteger bitAnd: 16rFFFF) = aSmallInteger)
		ifTrue: [ ^shortCluster ].

	^aSmallInteger > 0 
		ifTrue: [ positiveSmallIntegerCluster ]
		ifFalse: [ negativeSmallIntegerCluster ]! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/21/2011 01:08'!
visitSubstitution: anObject

	^hookSubstitutionCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:43'!
visitSymbol: aSymbol

	^symbolCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:43'!
visitSystemDictionary: anObject

	^wellKnownObjectCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:43'!
visitTrait: aTrait

	^traitCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:43'!
visitUndefinedObject: anObject

	^wellKnownObjectCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 03:04'!
visitVariableObject: anObject

	^variableObjectClusters
		at: anObject class
		ifAbsentPut: [ FLVariableObjectCluster for: anObject class ]! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/15/2011 23:18'!
visitWordsArrayedCollection: anArrayedCollection

	^wordArrayedCollectionsClusters
		at: anArrayedCollection class
		ifAbsentPut: [ FLWordArrayedCollectionCluster for: anArrayedCollection class ]! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 03:07'!
visitWordsObject: anObject

	^wordObjectClusters
		at: anObject class
		ifAbsentPut: [ FLWordObjectCluster for: anObject class ]! !

FLMapper subclass: #FLGlobalAssociationMapper
	instanceVariableNames: 'internalAssociations cluster'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Mappers'!

!FLGlobalAssociationMapper commentStamp: 'MartinDias 5/30/2011 01:15' prior: 0!
I map variable bindings that belong to Smalltalk global dictionary.

For example, 

	Smalltalk globals associationAt: #Transcript 
	
is mapped by me.!

!FLGlobalAssociationMapper methodsFor: 'accessing' stamp: 'MartinDias 5/30/2011 02:01'!
accepts: anObject

	^ anObject isVariableBinding and: [(self isGlobalAssociation: anObject) and: [ self isExternalAssociation: anObject ]].
! !

!FLGlobalAssociationMapper methodsFor: 'accessing' stamp: 'MartinDias 8/11/2011 02:34'!
clusterFor: anObject

	^cluster! !

!FLGlobalAssociationMapper methodsFor: 'initialize-release' stamp: 'MartinDias 8/11/2011 02:34'!
initialize

	super initialize.
	internalAssociations := OrderedCollection new.
	cluster := FLGlobalAssociationCluster new.! !

!FLGlobalAssociationMapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 7/6/2011 14:30'!
internalObjects: aCollection

	internalAssociations := aCollection collect: [:aGlobal | Smalltalk globals associationAt: aGlobal name ]! !

!FLGlobalAssociationMapper methodsFor: 'private' stamp: 'MartinDias 5/20/2011 04:10'!
isExternalAssociation: anObject

	^ (internalAssociations includes: anObject ) not! !

!FLGlobalAssociationMapper methodsFor: 'private' stamp: 'MartinDias 5/20/2011 04:13'!
isGlobalAssociation: anObject

	^ Smalltalk globals includesAssociation: anObject! !

FLMapper subclass: #FLGlobalBehaviorMapper
	instanceVariableNames: 'internalGlobals globalBehaviors classSideCluster instanceSideCluster'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Mappers'!

!FLGlobalBehaviorMapper commentStamp: 'MartinDias 8/6/2011 23:51' prior: 0!
I map classes that belong to Smalltalk global dictionary.

For example, Integer is mapped by me.!

!FLGlobalBehaviorMapper methodsFor: 'accessing' stamp: 'MartinDias 8/6/2011 23:56'!
accepts: anObject

	^(anObject isBehavior or: [ anObject isTrait ]) 
		and: [(self isGlobal: anObject) and: [self isExternal: anObject ]]! !

!FLGlobalBehaviorMapper methodsFor: 'accessing' stamp: 'MartinDias 8/11/2011 02:35'!
clusterFor: anObject

	^anObject isClassSide
		ifTrue: [ classSideCluster ]
		ifFalse: [ instanceSideCluster ]
! !

!FLGlobalBehaviorMapper methodsFor: 'initialize-release' stamp: 'MartinDias 8/20/2011 14:49'!
initialize

	super initialize.
	internalGlobals := OrderedCollection new.
	globalBehaviors := IdentitySet new.
	Smalltalk globals allClassesAndTraitsDo: [:each | globalBehaviors add: each].
	classSideCluster := FLReferencedClassSideCluster new.
	instanceSideCluster := FLGlobalClassCluster new.! !

!FLGlobalBehaviorMapper methodsFor: 'accessing' stamp: 'MartinDias 8/6/2011 20:34'!
internalObjects: anObject
	internalGlobals := anObject! !

!FLGlobalBehaviorMapper methodsFor: 'private' stamp: 'MartinDias 8/6/2011 23:28'!
isExternal: anObject

	^(internalGlobals includes: anObject instanceSide) not! !

!FLGlobalBehaviorMapper methodsFor: 'private' stamp: 'MartinDias 8/6/2011 23:31'!
isGlobal: anObject

	^globalBehaviors includes: anObject instanceSide! !

FLMapper subclass: #FLGlobalMapper
	instanceVariableNames: 'cluster'
	classVariableNames: 'Globals'
	poolDictionaries: ''
	category: 'Fuel-Core-Mappers'!

!FLGlobalMapper commentStamp: 'MartinDias 8/18/2011 20:46' prior: 0!
I map global objects that belong to Smalltalk dictionary.

For example, Transcript is mapped by me.!

!FLGlobalMapper classSide methodsFor: 'class initialization' stamp: 'MartinDias 8/18/2011 20:27'!
initialize

	self initializeGlobals ! !

!FLGlobalMapper classSide methodsFor: 'class initialization' stamp: 'MartinDias 8/18/2011 20:26'!
initializeGlobals

	Globals := IdentitySet new
		add: Display;
		add: World;
		add: Smalltalk;
		add: Transcript;
		add: SourceFiles;
		add: Processor;
		add: ImageImports;
		add: TextConstants;
		add: Undeclared;
		add: SystemOrganization;
		add: Sensor;
		add: ActiveWorld;
		add: ActiveHand;
		yourself! !

!FLGlobalMapper methodsFor: 'accessing' stamp: 'MartinDias 8/18/2011 20:29'!
accepts: anObject

	^Globals includes: anObject! !

!FLGlobalMapper methodsFor: 'accessing' stamp: 'MartinDias 8/11/2011 02:36'!
clusterFor: anObject

	^cluster! !

!FLGlobalMapper methodsFor: 'initialize-release' stamp: 'MartinDias 8/18/2011 20:29'!
initialize

	super initialize.
	cluster := FLGlobalCluster new.
		! !

FLMapper subclass: #FLLightGlobalMapper
	instanceVariableNames: 'classSideCluster instanceSideCluster globalCluster'
	classVariableNames: 'Globals'
	poolDictionaries: ''
	category: 'Fuel-Core-Mappers'!

!FLLightGlobalMapper commentStamp: 'MartinDias 8/20/2011 23:43' prior: 0!
I map classes, traits, and global objects that belong to Smalltalk dictionary. I am included in FLAnalyzer newLight behavior. 

I am a fast simplified replace to this four clusters included in FLAnaluzer newFull behavior: 
- FLGlobalMapper
- FLGlobalBehaviorMapper
- FLClassPoolAssociationMapper
- FLGlobalAssociationMapper 

For example, the global Transcript as well as any class in the image, are mapped by me.!

!FLLightGlobalMapper class methodsFor: 'initialization' stamp: 'MartinDias 8/18/2011 20:12'!
initialize

	self initializeGlobals! !

!FLLightGlobalMapper class methodsFor: 'initialization' stamp: 'MartinDias 8/18/2011 20:20'!
initializeGlobals

	Globals := IdentitySet new
		add: Display;
		add: World;
		add: Smalltalk;
		add: Transcript;
		add: SourceFiles;
		add: Processor;
		add: ImageImports;
		add: TextConstants;
		add: Undeclared;
		add: SystemOrganization;
		add: Sensor;
		add: ActiveWorld;
		add: ActiveHand;
		yourself! !

!FLLightGlobalMapper methodsFor: 'accessing' stamp: 'MartinDias 8/18/2011 20:19'!
chainedClusterFor: anObject
	"Uses the Chain of Responsibility pattern to answer the cluster which maps with the received object."

	(anObject isBehavior or: [ anObject isTrait ]) ifTrue: [
		^anObject isClassSide
			ifTrue: [ classSideCluster ]
			ifFalse: [ instanceSideCluster ]].

	(Globals includes: anObject) ifTrue: [ ^globalCluster ].
	
	^next chainedClusterFor: anObject.! !

!FLLightGlobalMapper methodsFor: 'initialize-release' stamp: 'MartinDias 8/18/2011 20:10'!
initialize

	super initialize.
	classSideCluster := FLReferencedClassSideCluster new.
	instanceSideCluster := FLGlobalClassCluster new.
	globalCluster := FLGlobalCluster new.! !

!FLMapper methodsFor: 'accessing' stamp: 'MartinDias 5/30/2011 02:01'!
accepts: anObject
	"Tells if the received object is handled by this analyzer"

	self subclassResponsibility ! !

!FLMapper methodsFor: 'accessing' stamp: 'MartinDias 8/7/2011 20:21'!
chainedClusterFor: anObject
	"Uses the Chain of Responsibility pattern to answer the cluster which maps with the received object."

	^(self accepts: anObject)
		ifTrue: [ self clusterFor: anObject ]
		ifFalse: [ next chainedClusterFor: anObject ].
! !

!FLMapper methodsFor: 'accessing' stamp: 'MartinDias 8/7/2011 20:19'!
clusterFor: anObject
	"Answers the cluster which maps with the received object."

	self subclassResponsibility ! !

!FLMapper methodsFor: 'accessing' stamp: 'MartinDias 8/6/2011 20:37'!
internalObjects: aCollection! !

!FLMapper methodsFor: 'accessing' stamp: 'MartinDias 8/7/2011 17:19'!
next
	^ next! !

!FLMapper methodsFor: 'accessing' stamp: 'MartinDias 8/7/2011 17:19'!
next: anObject
	next := anObject! !

FLMapper subclass: #FLSubstitutionMapper
	instanceVariableNames: 'condition substitutionCluster'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Mappers'!

!FLSubstitutionMapper commentStamp: 'MartinDias 8/20/2011 23:45' prior: 0!
I map substituted objects. 

For example, suppose you want to substitute instances of WriteStream by nil. In such case, I will map every WriteStream instance to my substitutions cluster, and its factory block will value to nil with any argument.
!

!FLSubstitutionMapper classSide methodsFor: 'instance creation' stamp: 'MartinDias 8/21/2011 01:28'!
when: aCondition substituteBy: aFactory

	^super basicNew
		condition: aCondition;
		substitutionCluster: (FLFactorySubstitutionCluster with: aFactory);
		initialize;
		yourself! !

!FLSubstitutionMapper methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 19:37'!
accepts: anObject

	^condition value: anObject! !

!FLSubstitutionMapper methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 20:31'!
clusterFor: anObject

	^substitutionCluster! !

!FLSubstitutionMapper methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 19:37'!
condition
	^ condition! !

!FLSubstitutionMapper methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 19:37'!
condition: anObject
	condition := anObject! !

!FLSubstitutionMapper methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 20:33'!
substitutionCluster
	^ substitutionCluster! !

!FLSubstitutionMapper methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 20:33'!
substitutionCluster: anObject
	substitutionCluster := anObject! !

Object subclass: #FLMaterializer
	instanceVariableNames: 'stream materializedInstances materializedRoot clusterCount indexCluster clusters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLMaterializer commentStamp: '<historical>' prior: 0!
I am a binary object materializer. 

By sending me #materialize, I will recreate previously serialized objects by FLSerializer from a byte stream. The results are obtained sending #materializedRoots.

!

!FLMaterializer class methodsFor: 'materializing' stamp: 'MarianoMartinezPeck 5/27/2011 11:04'!
materializeFromByteArray: byteArray
  	| materializer stream |
	stream := byteArray readStream.
    	materializer := self on: stream.
    	materializer materialize.
    	stream close.
    	^ materializer root
! !

!FLMaterializer class methodsFor: 'materializing' stamp: 'MarianoMartinezPeck 5/23/2011 23:07'!
materializeFromFileNamed: aFilename
  	| materializer aFileStream |
    	aFileStream := (StandardFileStream fileNamed: aFilename) binary.
    	materializer := self on: aFileStream.
    	materializer materialize.
    	aFileStream close.
    	^ materializer root
! !

!FLMaterializer class methodsFor: 'instance creation' stamp: 'MartinDias 9/20/2010 20:34'!
on: aReadStream

	^self new
		stream: aReadStream; 
		yourself
! !

!FLMaterializer methodsFor: 'accessing' stamp: 'MartinDias 10/13/2010 11:31'!
addAllMaterialized: aCollection
	
	materializedInstances addAll: aCollection ! !

!FLMaterializer methodsFor: 'private' stamp: 'MartinDias 8/27/2011 23:10'!
instancesStep

	clusterCount timesRepeat: [ self materializeCluster ]
! !

!FLMaterializer methodsFor: 'materializing' stamp: 'MartinDias 8/27/2011 23:10'!
materialize

	self materializeHeader.
	self instancesStep.
	self referencesStep.
	self postReferencesStep.
	self materializeRoot.
	^ self root! !

!FLMaterializer methodsFor: 'private' stamp: 'MartinDias 8/27/2011 22:37'!
materializeCluster

	| aCluster |
	aCluster := self nextEncodedCluster.
	aCluster materializeInstancesStepWith: self.
	clusters add: aCluster.! !

!FLMaterializer methodsFor: 'private' stamp: 'MartinDias 8/27/2011 20:01'!
materializeHeader

	indexCluster := self nextEncodedCluster.

	materializedInstances := OrderedCollection new: self nextEncodedInteger.
	materializedInstances resetTo: 1. "Hack that avoids OrderedCollection>>makeRoomAtLast"

	clusterCount := self nextEncodedInteger.
	clusters := OrderedCollection new: clusterCount.
	clusters resetTo: 1. "Hack that avoids OrderedCollection>>makeRoomAtLast"
! !

!FLMaterializer methodsFor: 'private' stamp: 'MartinDias 6/3/2011 21:16'!
materializeRoot

	materializedRoot := self nextEncodedReference! !

!FLMaterializer methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 7/1/2011 09:54'!
materializedInstances
	^ materializedInstances ! !

!FLMaterializer methodsFor: 'decoding' stamp: 'MartinDias 8/10/2011 03:18'!
nextEncodedCluster

	^(Smalltalk classNamed: stream nextString) new! !

!FLMaterializer methodsFor: 'decoding' stamp: 'MartinDias 8/11/2011 02:46'!
nextEncodedInteger
	
	^FLPositiveSmallIntegerCluster new materializeFrom: stream! !

!FLMaterializer methodsFor: 'decoding' stamp: 'MartinDias 6/3/2011 21:16'!
nextEncodedReference 

	^ materializedInstances at: 
		(indexCluster materializeFrom: stream)! !

!FLMaterializer methodsFor: 'materializing' stamp: 'MartinDias 8/27/2011 22:49'!
postReferencesStep

	clusters do: [ :aCluster | aCluster materializePostReferencesStepWith: self ]! !

!FLMaterializer methodsFor: 'materializing' stamp: 'MartinDias 8/27/2011 22:52'!
referencesStep

	clusters do: [ :aCluster | aCluster materializeReferencesStepWith: self ]! !

!FLMaterializer methodsFor: 'accessing' stamp: 'MartinDias 10/28/2010 10:51'!
root

	^ materializedRoot ! !

!FLMaterializer methodsFor: 'accessing' stamp: 'MartinDias 10/6/2010 19:50'!
stream
	^ stream! !

!FLMaterializer methodsFor: 'accessing' stamp: 'sd 9/8/2010 15:23'!
stream: aStream
	stream:= aStream.! !

Object subclass: #FLSerializer
	instanceVariableNames: 'stream instanceIndexes originalStream indexCluster root analyzer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLSerializer commentStamp: '<historical>' prior: 0!
I am a binary object serializer. 

By sending me #serializeAll: with a collection of objects as argument, I will write them on a byte stream, such that later a FLMaterializer instance will be able to read the stream and recreate the objects in the same or another image.!

!FLSerializer class methodsFor: 'instance creation' stamp: 'MartinDias 5/30/2011 02:55'!
newInMemorySizeHint: sizeHint

	^ self new
		initializeInMemoryWith: sizeHint;
		yourself.! !

!FLSerializer class methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 6/3/2011 14:25'!
on: aWriteStream

	self flag: #todo "this 4096 should be a setting".
	^ self on: aWriteStream bufferSize: 4096! !

!FLSerializer class methodsFor: 'instance creation' stamp: 'MartinDias 8/20/2011 17:30'!
on: aWriteStream bufferSize: aBufferSize
	
	^ self new
		initializeOn: aWriteStream 
		bufferSize: aBufferSize;
		yourself.
	! !

!FLSerializer class methodsFor: 'serializing' stamp: 'MartinDias 5/30/2011 02:54'!
serialize: anObject toFileNamed: aFilename

	| serializer aFileStream |
 	aFileStream := (StandardFileStream fileNamed: aFilename) binary.
   	serializer := self on: aFileStream.
    	serializer serialize: anObject.
    	aFileStream close.
	! !

!FLSerializer class methodsFor: 'serializing' stamp: 'MartinDias 5/30/2011 02:54'!
serializeInMemory: anObject
	"No stream is needed by the user. An internal in-memory stream will be used. This method returns a ByteArray representing the serialization"
	
	^ (self newInMemorySizeHint: 100)
		serializeAndGetContents: anObject! !

!FLSerializer methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 17:33'!
analyzer
	^ analyzer! !

!FLSerializer methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 17:33'!
analyzer: anObject
	analyzer := anObject! !

!FLSerializer methodsFor: 'encoding' stamp: 'MartinDias 8/10/2011 03:16'!
encodeIdOf: aCluster

	stream nextStringPut: aCluster class name! !

!FLSerializer methodsFor: 'encoding' stamp: 'MartinDias 8/11/2011 02:45'!
encodeInteger: anInteger

	FLPositiveSmallIntegerCluster new 
		serialize: anInteger
		on: stream! !

!FLSerializer methodsFor: 'encoding' stamp: 'MartinDias 6/3/2011 20:42'!
encodeReferenceTo: anObject

	indexCluster 
		serialize: (instanceIndexes at: anObject) 
		on: stream! !

!FLSerializer methodsFor: 'initialize-release' stamp: 'MartinDias 8/27/2011 19:43'!
initialize

	super initialize.
	analyzer := FLAnalyzer newDefault! !

!FLSerializer methodsFor: 'initialize-release' stamp: 'MartinDias 8/20/2011 17:29'!
initializeInMemoryWith: sizeHint

	"Usually, pre-allocating a collection is faster than make it grow. So we can have a default initial value"
	instanceIndexes := IdentityDictionary new: sizeHint.
	originalStream := ByteArray new writeStream.
	stream := originalStream! !

!FLSerializer methodsFor: 'initialize-release' stamp: 'MartinDias 8/20/2011 17:29'!
initializeOn: aWriteStream bufferSize: aBufferSize

	originalStream := aWriteStream.
	self flag: #todo. "when the settings is ready, we can direcly set to 'stream' the original one instead of this buffer'  "
	stream := FLBufferedWriteStream on: originalStream.
	stream sizeBuffer: aBufferSize.! !

!FLSerializer methodsFor: 'private' stamp: 'MartinDias 8/27/2011 23:09'!
instancesStepOfAll: clusters

	clusters do: [ :aCluster | self serializeInstancesOf: aCluster ]! !

!FLSerializer methodsFor: 'private' stamp: 'MartinDias 8/27/2011 22:35'!
postReferencesStepOfAll: clusters

	clusters do: [ :aCluster | aCluster serializePostReferencesStepWith: self ]! !

!FLSerializer methodsFor: 'private' stamp: 'MartinDias 8/27/2011 22:36'!
referencesStepOfAll: clusters

	clusters do: [ :aCluster | aCluster serializeReferencesStepWith: self ]! !

!FLSerializer methodsFor: 'private' stamp: 'MartinDias 8/22/2011 05:52'!
registerIndexesOfAll: clusters

	clusters do: [ :aCluster | aCluster registerIndexesOn: instanceIndexes ]! !

!FLSerializer methodsFor: 'serializing' stamp: 'MartinDias 8/27/2011 23:09'!
serialize: anObject
	"Main message for serializing"

	| objectCount clusters aClusterization |
	aClusterization := analyzer clusterizationOf: anObject.

	clusters := aClusterization clusters.
	objectCount := aClusterization objectCount.
	instanceIndexes := IdentityDictionary new: objectCount.
	indexCluster := FLSmallIntegerCluster for: objectCount.
	root := anObject.

	self serializeHeaderWith: clusters and: objectCount.
	self registerIndexesOfAll: clusters.  
	self instancesStepOfAll: clusters.
	self referencesStepOfAll: clusters.
	self postReferencesStepOfAll: clusters.
	self serializeRoot.
	root := nil.

	stream flush.
	self flag: #fix.  "this is an ugly hack beause benchmarks and test are wrong and reuses the same stream for serializing and maerializing"
	stream := originalStream.
	! !

!FLSerializer methodsFor: 'serializing' stamp: 'MartinDias 8/20/2011 17:21'!
serializeAndGetContents: anObject 
	"Serializes using the default analyzer"

	self serialize: anObject.
	^ stream contents! !

!FLSerializer methodsFor: 'private' stamp: 'MartinDias 8/20/2011 17:09'!
serializeHeaderWith: clusters and: objectCount

	self encodeIdOf: indexCluster.
	self encodeInteger: objectCount.
	self encodeInteger: clusters size! !

!FLSerializer methodsFor: 'private' stamp: 'MartinDias 8/27/2011 22:35'!
serializeInstancesOf: aCluster

	self encodeIdOf: aCluster.
	aCluster serializeInstancesStepWith: self.
! !

!FLSerializer methodsFor: 'private' stamp: 'MarianoMartinezPeck 8/3/2011 18:43'!
serializeRoot

	self encodeReferenceTo: root! !

!FLSerializer methodsFor: 'accessing' stamp: 'sd 9/8/2010 14:59'!
stream
	^ stream! !

Object subclass: #FLSimpleStack
	instanceVariableNames: 'array slotIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Collections'!

!FLSimpleStack commentStamp: 'MarianoMartinezPeck 6/5/2011 12:29' prior: 0!
FLSimpleStack is a simple stack used in Fuel to avoid a recursion while traversing the graph to serialize. While analizing the content of an object (inst vars) we can encounter simple objects (direct string representation) or complex objects (composite). In the latter case we start analizing those instVars.

So... Imagine object X with two instVars Y and Z. Imagine the method of the traverse is called #analye:.  So you do #analize: X.  Then, in such method you check whether X has regular pointers to regular objects, and if true, you analize them. So in this case you would send #analyze: Y and #analize: Z, generating the recursion. Right ?  Notice that the time between analyse: Y and analyse: Z is dependent on the graph that is attached to Y. Y can have objects that have further objects etc. So leaving X aside, Y can be the first object to serialize and Z the last.

With this FLSimpleStack, instead of sending #analyze: Y and #analize: Z  what Fuel does is just to do a push on a stack:   #push: Y  and #push: Z. And then we are done with X, so we pop from the stack and we continue with the next object (at some point in the future we will pop Y and Z) When there are no more objects in the stack it means we are done. !

!FLSimpleStack class methodsFor: 'instance creation' stamp: 'MartinDias 5/15/2011 23:11'!
hintSize

	^64! !

!FLSimpleStack class methodsFor: 'instance creation' stamp: 'MartinDias 5/15/2011 23:11'!
new
	
	^ self new: self hintSize! !

!FLSimpleStack class methodsFor: 'instance creation' stamp: 'MartinDias 5/15/2011 23:09'!
new: nElements
	"Create a Set large enough to hold nElements without growing"
	^ self basicNew initialize: nElements! !

!FLSimpleStack methodsFor: 'accessing' stamp: 'MartinDias 5/15/2011 23:08'!
capacity

	^ array size! !

!FLSimpleStack methodsFor: 'adding' stamp: 'MartinDias 5/15/2011 23:15'!
grow
	"Become larger in capacity."
	| newArray |
	newArray := Array new: array size * 2.
	newArray replaceFrom: 1 to: array size with: array startingAt: 1.
	array := newArray! !

!FLSimpleStack methodsFor: 'initialize-release' stamp: 'MartinDias 5/15/2011 23:10'!
initialize: hintSize

	array := Array new: hintSize.
	slotIndex := 1.! !

!FLSimpleStack methodsFor: 'accessing' stamp: 'MartinDias 5/15/2011 22:54'!
isEmpty

	^ 1 = slotIndex! !

!FLSimpleStack methodsFor: 'removing' stamp: 'MartinDias 5/15/2011 22:52'!
pop
	"Returns the first element and remove it from the stack."

	slotIndex := slotIndex - 1.	
	^ array at: slotIndex ! !

!FLSimpleStack methodsFor: 'printing' stamp: 'MartinDias 6/3/2011 20:25'!
printOn: aStream 
	"Append a sequence of characters that identify the receiver to aStream."

	super printOn: aStream.
	array printElementsOn: aStream
! !

!FLSimpleStack methodsFor: 'adding' stamp: 'MartinDias 5/15/2011 22:44'!
push: anObject 
	"Adds a new object of any kind on top of the stack."

	array at: slotIndex put: anObject.
	array size = slotIndex ifTrue: [ self grow ].
	slotIndex := slotIndex + 1.
	^ anObject.! !

Object subclass: #FLVariablesMapping
	instanceVariableNames: 'mapping theClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLVariablesMapping commentStamp: 'MartinDias 8/1/2011 03:01' prior: 0!
I am used to materialize instance variables in an object, tolerating "class shape changing".

Cases tolerated are:
- instance variable added
- instance variable order change
- instance variable removed
!

!FLVariablesMapping class methodsFor: 'instance creation' stamp: 'MartinDias 7/27/2011 20:29'!
for: aClass
	
	^ self basicNew
		theClass: aClass;
		yourself.! !

!FLVariablesMapping methodsFor: 'private' stamp: 'MartinDias 7/27/2011 23:21'!
instanceVariableNamesToSerialize

	| ignoredInstanceVariableNames instanceVariableNamesToSerialize |
	ignoredInstanceVariableNames := theClass fuelIgnoredInstanceVariableNames.
	instanceVariableNamesToSerialize := OrderedCollection new.

	theClass fuelInstVarNamesAndOffsetsDo: [:name :index | 
		(ignoredInstanceVariableNames includes: name) 
			ifFalse: [ instanceVariableNamesToSerialize add: name ]].

	^ instanceVariableNamesToSerialize! !

!FLVariablesMapping methodsFor: 'accessing' stamp: 'MartinDias 7/27/2011 20:30'!
mapping
	^ mapping! !

!FLVariablesMapping methodsFor: 'accessing' stamp: 'MartinDias 7/27/2011 20:30'!
mapping: anObject
	mapping := anObject! !

!FLVariablesMapping methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/27/2011 23:22'!
materializeMappingFrom: aStream

	| instanceVariableNamesSerialized |
	instanceVariableNamesSerialized := (1 to: aStream next) collect: [ :index | aStream nextString ].

	mapping := instanceVariableNamesSerialized  
		collect: [ :name | theClass instVarIndexFor: name ifAbsent: [ nil ] ].

! !

!FLVariablesMapping methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/27/2011 20:37'!
materializeReferencesOf: anObject with: aMaterializer 

	mapping do: [ :index | 
			| reference |
			reference := aMaterializer nextEncodedReference.
			index ifNotNil: [
				anObject 
					instVarAt: index
					put: reference ]].! !

!FLVariablesMapping methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/27/2011 23:22'!
serializeMappingOn: aStream

	| instanceVariableNamesToSerialize |
	instanceVariableNamesToSerialize := self instanceVariableNamesToSerialize.

	aStream nextPut: instanceVariableNamesToSerialize size.	
	instanceVariableNamesToSerialize do: [ :name | aStream nextStringPut: name ].

	mapping := instanceVariableNamesToSerialize 
		collect: [ :name | theClass instVarIndexFor: name ].! !

!FLVariablesMapping methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/27/2011 20:02'!
serializeReferencesOf: anObject with: aSerializer 

	mapping do: [ :index | 
		aSerializer encodeReferenceTo: (anObject instVarAt: index) ].! !

!FLVariablesMapping methodsFor: 'accessing' stamp: 'MartinDias 7/27/2011 20:30'!
theClass
	^ theClass! !

!FLVariablesMapping methodsFor: 'accessing' stamp: 'MartinDias 7/27/2011 20:30'!
theClass: anObject
	theClass := anObject! !

Object subclass: #FuelUtilities
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Utilities'!

!FuelUtilities commentStamp: 'MarianoMartinezPeck 6/5/2011 12:25' prior: 0!
FuelUtilities is a class that contains utilities when developing Fuel. Right now the only usage is to do some cleanings. Sometimes updating versions from Monticello bring problems with traits or things like that. Right now the only usage is:

FuelUtilities cleanUpEverything.
!

!FuelUtilities classSide methodsFor: 'cleaning' stamp: 'MartinDias 8/11/2011 02:46'!
cleanUpEverything
	"Sometimes when updating Fuel versions Monticello gets stupid and we need to do some cleanups."
	Smalltalk garbageCollect.
	Smalltalk fixObsoleteReferences.

	(PackageInfo named: 'Fuel') classes,
	(PackageInfo named: 'FuelFameExtension') classes,
	(PackageInfo named: 'FuelContainer') classes,
	(PackageInfo named: 'FuelMooseExtension') classes
	    do: [:c | c compileAll ].
	((PackageInfo named: 'Fuel') classes select: [:each | each isBehavior ]) do: [:each | each initialize]
! !

!Object methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:43'!
fuelAccept: aVisitor

	self class isFixed
		ifTrue: [ ^aVisitor visitFixedObject: self ].
	self class isPointers
		ifTrue: [ ^aVisitor visitVariableObject: self ].
	self class isBytes
		ifTrue: [ ^aVisitor visitBytesObject: self ].
	self class isWords
		ifTrue: [	 ^aVisitor visitWordsObject: self ].
	self error: 'Something is wrong!!'! !

!Object methodsFor: '*Fuel' stamp: 'MartinDias 8/27/2011 19:03'!
fuelAfterMaterialization
	"Materialization process will send this message after materializing instances of my class"! !

!Object methodsFor: '*Fuel' stamp: 'MartinDias 7/17/2011 18:47'!
restoreEndianness
	"If I am a word object, correct endianess because I was just read in from a stream."

	"^ self"
! !
FLLightGlobalMapper initialize!
FLClassCluster initialize!
FLGlobalMapper initialize!
FLMetaclassCluster initialize!
FLWellKnownObjectsCluster initialize!
